#!/usr/bin/env python
"""
Strain Mapping Launcher
"""

from qtpy.QtWidgets import QMainWindow

from ibeatles import FileType, load_ui
from ibeatles.step6 import ParametersToDisplay
from ibeatles.step6.display import Display
from ibeatles.step6.event_handler import EventHandler
from ibeatles.step6.export import Export
from ibeatles.step6.get import Get
from ibeatles.step6.initialization import Initialization
from ibeatles.utilities.status_message_config import (
    StatusMessageStatus,
    show_status_message,
)
from ibeatles.widgets.qrangeslider import FakeKey


class StrainMappingLauncher:
    def __init__(self, parent=None, fitting_parent=None):
        self.parent = parent

        # if self.parent.fitting_ui is None:
        #     # show_status_message(
        #     #     parent=fitting_parent,
        #     #     message="Strain Mapping requires to first launch the fitting window!",
        #     #     status=StatusMessageStatus.error,
        #     #     duration_s=10,
        #     # )
        #     show_status_message(
        #         parent=self.parent,
        #         message="Strain Mapping requires to first launch the fitting window!",
        #         status=StatusMessageStatus.error,
        #         duration_s=10,
        #     )
        # else:
        try:
            strain_mapping_window = StrainMappingWindow(parent=parent)
            strain_mapping_window.show()
            strain_mapping_window.ui.range_slider.keyPressEvent(FakeKey(key="down"))
            self.parent.strain_mapping_ui = strain_mapping_window
        except ValueError:
            # show_status_message(
            #     parent=fitting_parent,
            #     message="Please perform a fitting first",
            #     status=StatusMessageStatus.error,
            #     duration_s=10,
            # )
            show_status_message(
                parent=self.parent,
                message="Please perform a fitting first",
                status=StatusMessageStatus.error,
                duration_s=10,
            )


class StrainMappingWindow(QMainWindow):
    # slider_nbr_steps = 1000
    slider_min = 0
    slider_max = 1000

    integrated_image = None
    image_size = {"width": None, "height": None}

    # min_max = {'d': {min: -1,
    #                  max: -1},
    #            'strain_mapping': {min: -1,
    #                               max: -1},
    #            }
    min_max = {
        ParametersToDisplay.d: {"min": None, "max": None},
        ParametersToDisplay.strain_mapping: {"min": None, "max": None},
    }

    histogram = {"d": None, "strain_mapping": None, "integrated_image": None}

    previous_parameters_displayed = ParametersToDisplay.d

    colorbar = None

    bin_size = None
    d_array = None
    compact_d_array = None
    strain_mapping_array = None
    compact_strain_mapping_array = None

    def __init__(self, parent=None):
        self.parent = parent
        QMainWindow.__init__(self, parent=parent)
        self.ui = load_ui("ui_strainMapping.ui", baseinstance=self)
        self.setWindowTitle("6. Strain Mapping")

        o_init = Initialization(parent=self, grand_parent=self.parent)
        o_init.all()

        o_event = EventHandler(parent=self, grand_parent=self.parent)
        o_event.process_data()

        o_init.min_max_values()
        o_init.range_slider()

        self.update_display()

        o_get = Get(parent=self)
        self.previous_parameter_displayed = o_get.parameter_to_display()
        self.update_min_max_values()

    ## menu

    # export table
    def export_table_ascii(self):
        """export table only in ASCII"""
        o_export = Export(parent=self, grand_parent=self.parent)
        o_export.table()

    def export_table_json(self):
        """export table only in JSON"""
        o_export = Export(parent=self, grand_parent=self.parent)
        o_export.table(file_type=FileType.json)

    # export images
    def export_images_all_tiff(self):
        """export d, strain mapping and integrated in TIFF"""
        o_export = Export(parent=self, grand_parent=self.parent)
        o_export.image(d_spacing_image=True, strain_mapping_image=True, integrated_image=True)

    def export_images_d_tiff(self):
        """export d in TIFF"""
        o_export = Export(parent=self, grand_parent=self.parent)
        o_export.image(d_spacing_image=True)

    def export_images_strain_mapping_tiff(self):
        """export strain mapping in TIFF"""
        o_export = Export(parent=self, grand_parent=self.parent)
        o_export.image(strain_mapping_image=True)

    def export_images_integrated_tiff(self):
        """export integrated image in TIFF"""
        o_export = Export(parent=self, grand_parent=self.parent)
        o_export.image(integrated_image=True)

    # export table and images
    def export_all_ascii_json_tiff(self):
        """export table (ASCII and JSON) and all images as TIFF"""
        o_export = Export(parent=self, grand_parent=self.parent)
        output_folder = o_export.select_output_folder()
        o_export.image(
            d_spacing_image=True,
            strain_mapping_image=True,
            integrated_image=True,
            output_folder=output_folder,
        )
        o_export.table(file_type=FileType.ascii, output_folder=output_folder)
        o_export.table(file_type=FileType.json, output_folder=output_folder)

    def export_all_hdf5(self):
        """export table and images in HDF5"""
        o_export = Export(parent=self, grand_parent=self.parent)
        output_folder = o_export.select_output_folder()
        o_export.hdf5(output_folder=output_folder)

    def action_configuration_for_cli_clicked(self):
        print("action clicked!")
        o_export = Export(parent=self, grand_parent=self.parent)
        output_folder = o_export.select_output_folder()
        o_export.config_for_cli(output_folder=output_folder)

    def fitting_algorithm_changed(self):
        self.update_display()

    def interpolation_cmap_method_changed(self, _):
        self.ui.matplotlib_plot.axes.cla()
        self.ui.matplotlib_plot.draw()
        o_event = EventHandler(parent=self, grand_parent=self.parent)
        o_event.interpolation_cmap_method_changed()

    def parameters_to_display_changed(self):
        self.update_min_max_values()
        self.update_display()

    def d0_to_use_changed(self):
        self.update_display()
        self.update_slider_and_lineEdit()

    def min_max_value_changed(self):
        o_event = EventHandler(parent=self, grand_parent=self.parent)
        o_event.min_max_changed()

    def update_slider_and_lineEdit(self):
        self.update_min_max_values()
        o_get = Get(parent=self)
        parameter_displayed = o_get.parameter_to_display()
        min_value = self.min_max[parameter_displayed]["global_min"]
        max_value = self.min_max[parameter_displayed]["global_max"]
        self.ui.max_range_lineEdit.setText(f"{max_value:.5f}")
        self.ui.min_range_lineEdit.setText(f"{min_value:.5f}")

    def min_max_lineEdit_value_changed(self):
        min_value = float(self.ui.min_range_lineEdit.text())
        max_value = float(self.ui.max_range_lineEdit.text())
        o_get = Get(parent=self)
        parameter_displayed = o_get.parameter_to_display()
        self.min_max[parameter_displayed]["global_min"] = min_value
        self.min_max[parameter_displayed]["global_max"] = max_value

        if self.min_max[parameter_displayed]["min"] < min_value:
            self.min_max[parameter_displayed]["min"] = min_value

        if self.min_max[parameter_displayed]["max"] > max_value:
            self.min_max[parameter_displayed]["max"] = max_value

        self.update_min_max_values()
        self.update_display()

    def update_display(self):
        o_display = Display(parent=self, grand_parent=self.parent)
        o_display.run()

    def calculate_int_value_from_real(self, float_value=0, max_float=0, min_float=0):
        """
        use the real value to return the int value (between 0 and 100) to use in the slider
        Parameters
        ----------
        float_value

        Returns
        -------
        """
        term1 = (float_value - min_float) / (max_float - min_float)
        term2 = int(round(term1 * (self.slider_max - self.slider_min)))
        return term2

    def update_min_max_values(self):
        o_get = Get(parent=self)
        parameter_displayed = o_get.parameter_to_display()

        global_min_value = self.min_max[parameter_displayed]["global_min"]
        global_max_value = self.min_max[parameter_displayed]["global_max"]

        self.ui.range_slider.setRealMin(global_min_value)
        self.ui.range_slider.setRealMax(global_max_value)

        self.ui.max_range_lineEdit.setText(f"{global_max_value:.5f}")
        self.ui.min_range_lineEdit.setText(f"{global_min_value:.5f}")

        self.ui.range_slider.setFocus(True)

    def range_slider_start_value_changed(self, value):
        real_start_value = self.ui.range_slider.get_real_value_from_slider_value(value)
        o_get = Get(parent=self)
        parameters_to_display = o_get.parameter_to_display()
        self.min_max[parameters_to_display]["max"] = real_start_value
        self.min_max_value_changed()

    def range_slider_end_value_changed(self, value):
        real_end_value = self.ui.range_slider.get_real_value_from_slider_value(value)
        o_get = Get(parent=self)
        parameters_to_display = o_get.parameter_to_display()
        self.min_max[parameters_to_display]["min"] = real_end_value
        self.min_max_value_changed()

    def display_hidden_plot_changed(self):
        if self.ui.checkBox.isChecked():
            self.ui.stackedWidget.setCurrentIndex(0)
        else:
            self.ui.stackedWidget.setCurrentIndex(1)



#!/usr/bin/env python
"""
Reduction tools
"""

import logging

import numpy as np
import scipy.ndimage

from ibeatles.step2 import KernelType


def moving_average(data=None, kernel_type=KernelType.gaussian, kernel=None):
    """
    if both box_kernel and gaussian_kernel are provided, box_kernel will be used by default
    Parameters
    ----------
    data: if 3D, dimensions must be given as followed (x, y, tof)
    kernel_type: KernelType.box or KernelType.gaussian
    kernel: size of kernel

    Raises
    ------
    ValueError if input_array is not a 2 or 3D array
    Returns
    -------
    """

    if data is None:
        raise ValueError("Provide a signal")

    if len(np.shape(data)) == 1:
        raise ValueError("Data must be 2D image or 3D volume, or TOF stack of 2D images.")

    if len(np.shape(data)) > 3:
        raise ValueError("Data must be 2D image or 3D volume, or TOF stack of 2D images.")

    if kernel is None:
        raise ValueError("You need to provide a kernel!")

    if len(kernel) == 1:
        raise ValueError("Kernel must be at least of size 2.")

    if (len(np.shape(data)) == 2) and (len(kernel) == 3):
        raise ValueError("Data is 2d but filtering kernel is 3D.")

    # TOF data (3D), 2D kernel
    if len(np.shape(data)) == 3 and (len(kernel) == 2):
        logging.info(
            "-> Data is 3D but filtering kernel is 2D. Applying filter to each slice of the data (third dimension)."
        )

        outsignal = np.zeros((np.shape(data)[0], np.shape(data)[1], np.shape(data)[2]))

        if kernel_type == KernelType.box:
            kernel = np.ones((kernel[0], kernel[1]))
            kernel = kernel / np.sum(np.ravel(kernel))
            for i in range(0, np.shape(data)[2]):
                outsignal[:, :, i] = scipy.ndimage.convolve(data[:, :, i], kernel)
            return outsignal

        elif kernel_type == KernelType.gaussian:
            for i in range(0, np.shape(data)[2]):
                outsignal[:, :, i] = scipy.ndimage.gaussian_filter(data[:, :, i], kernel)

            return outsignal

        else:
            raise ValueError(f"Kernel Type {kernel_type} not supported! ")

    # TOF data (3D), 3D kernel
    elif len(np.shape(data)) == 3 and (len(kernel) == 3):
        logging.info("-> Data and filtering kernel are 3D. Applying 3D filter convolution.")

        if kernel_type == KernelType.box:
            kernel = np.ones((kernel[0], kernel[1], kernel[2]))
            kernel = kernel / np.sum(np.ravel(kernel))
            outsignal = scipy.ndimage.convolve(data, kernel)
            return outsignal

        elif kernel_type == KernelType.gaussian:
            outsignal = scipy.ndimage.gaussian_filter(data, kernel)
            return outsignal

        else:
            raise ValueError(f"Kernel Type {kernel_type} not supported! ")

    # image data (2D), 2D kernel
    elif len(np.shape(data)) == 2 and (len(kernel) == 2):
        logging.info("-> Data and filtering kernel are 2D. Applying 2D filter convolution.")

        if kernel_type == KernelType.box:
            kernel = np.ones((kernel[0], kernel[1]))
            kernel = kernel / np.sum(np.ravel(kernel))
            outsignal = scipy.ndimage.convolve(data, kernel)
            return outsignal

        elif kernel_type == KernelType.gaussian:
            outsignal = scipy.ndimage.gaussian_filter(data, kernel)
            return outsignal

        else:
            raise ValueError(f"Kernel Type {kernel_type} not supported! ")

    return



#!/usr/bin/env python
"""Model for Normalization Settings"""

import logging
from typing import Any, Dict

from ibeatles.core.config import (
    KernelSize,
    KernelType,
    NormalizationConfig,
    ProcessOrder,
    SampleBackground,
)


class NormalizationSettingsModel:
    """
    Model class for handling normalization settings.

    This class manages the configuration for normalization settings, including
    moving average parameters and sample background. It provides methods to
    update settings, convert between new and old configuration formats, and
    modify individual settings.
    """

    def __init__(self):
        """Initialize the NormalizationSettingsModel with default configuration."""
        self.config = NormalizationConfig()
        self._old_config: Dict[str, Any] = {}

    def update_from_config(self, config: NormalizationConfig):
        """
        Update the model's configuration from a NormalizationConfig object.

        Parameters
        ----------
        config : NormalizationConfig
            The configuration object to update from.
        """
        self.config = config

    def update_from_old_config(self, old_config: Dict[str, Any]):
        """
        Update the model's configuration from the old dictionary-based format.

        Parameters
        ----------
        old_config: Dict[str, Any]
            The old configuration dictionary.
        """
        self._old_config = old_config
        logging.debug("Deprecation warning: Using old configuration method. This will be removed in a future version.")

        # Convert old config to new config
        self.config.moving_average.active = old_config.get("activate", True)
        self.config.moving_average.dimension = "3D" if old_config.get("dimension") == "3d" else "2D"

        # Create KernelSize object
        kernel_size = {
            "y": old_config["size"].get("y", 3),
            "x": old_config["size"].get("x", 3),
        }
        if old_config.get("dimension") == "3d":
            kernel_size["lambda_"] = old_config["size"].get("l", 3)

        self.config.moving_average.size = KernelSize(**kernel_size)

        self.config.moving_average.type = (
            KernelType.gaussian if old_config.get("type") == "gaussian" else KernelType.box
        )
        self.config.processing_order = (
            ProcessOrder.moving_average_normalization
            if old_config.get("process order") == "option1"
            else ProcessOrder.normalization_moving_average
        )

    def get_old_config(self) -> Dict[str, Any]:
        """
        Convert the current configuration to the old dictionary-based format.

        Returns
        -------
        Dict[str, Any]
            The configuration in the old format.
        """
        default_size = KernelSize()
        is_default_size = (
            self.config.moving_average.size.y == default_size.y
            and self.config.moving_average.size.x == default_size.x
            and (
                self.config.moving_average.dimension == "2D"
                or self.config.moving_average.size.lambda_ == default_size.lambda_
            )
        )

        old_config = {
            "activate": self.config.moving_average.active,
            "dimension": "3d" if self.config.moving_average.dimension == "3D" else "2d",
            "size": {
                "flag": "default" if is_default_size else "custom",
                "y": self.config.moving_average.size.y,
                "x": self.config.moving_average.size.x,
                "l": default_size.lambda_,  # the old method requires this entry even if not in use
            },
            "type": "gaussian" if self.config.moving_average.type == KernelType.gaussian else "box",
            "process order": "option1"
            if self.config.processing_order == ProcessOrder.moving_average_normalization
            else "option2",
        }

        if self.config.moving_average.dimension == "3D":
            old_config["size"]["l"] = self.config.moving_average.size.lambda_

        return old_config

    def set_moving_average_active(self, active: bool):
        """
        Set the active state of moving average.

        Parameters
        ----------
        active: bool
            True to activate moving average, False to deactivate.
        """
        self.config.moving_average.active = active

    def set_dimension(self, dimension: str):
        """
        Set the dimension for moving average.

        Parameters
        ----------
        dimension: str
            The dimension, either "2D" or "3D".
        """
        self.config.moving_average.dimension = dimension
        if dimension == "2D" and "lambda" in self.config.moving_average.size:
            del self.config.moving_average.size["lambda"]
        elif dimension == "3D" and "lambda" not in self.config.moving_average.size:
            self.config.moving_average.size["lambda"] = 3

    def set_kernel_size(self, size: Dict[str, int]):
        """
        Set the kernel size for moving average.

        Parameters
        ----------
        size: Dict[str, int]
            A dictionary with 'y', 'x', and 'lambda' keys.
        """
        self.config.moving_average.size = size

    def set_kernel_type(self, kernel_type: KernelType):
        """
        Set the kernel type for moving average.

        Parameters
        ----------
        kernel_type: KernelType
            The type of kernel to use.
        """
        self.config.moving_average.type = kernel_type

    def set_processing_order(self, order: ProcessOrder):
        """
        Set the processing order.

        Parameters
        ----------
        order: ProcessOrder
            The order of processing steps.
        """
        self.config.processing_order = order

    def add_sample_background(self, background: SampleBackground):
        """
        Add a sample background to the configuration.

        Parameters
        ----------
        background: SampleBackground
            The sample background to add.
        """
        if self.config.sample_background is None:
            self.config.sample_background = []
        self.config.sample_background.append(background)

    def remove_sample_background(self, index: int):
        """
        Remove a sample background from the configuration.

        Parameters
        ----------
        index: int
            The index of the sample background to remove.
        """
        if self.config.sample_background and 0 <= index < len(self.config.sample_background):
            del self.config.sample_background[index]

    def update_sample_background(self, index: int, background: SampleBackground):
        """
        Update a sample background in the configuration.

        Parameters
        ----------
        index: int
            The index of the sample background to update.
        background: SampleBackground
            The new sample background data.
        """
        if self.config.sample_background and 0 <= index < len(self.config.sample_background):
            self.config.sample_background[index] = background

    def get_config(self) -> NormalizationConfig:
        """
        Get the current configuration.

        Returns
        -------
        NormalizationConfig
            The current normalization configuration.
        """
        return self.config



#!/usr/bin/env python
"""
Display module
"""

import pyqtgraph as pg

from ibeatles.utilities.math_tools import is_float, is_nan


class Display:
    def __init__(self, ui=None):
        self.ui = ui

    def vertical_line(
        self,
        x_position=0,
        item=None,
        label="\u03bb\u2080",
        pen=pg.mkPen(color="b", width=1.5),
    ):
        if item:
            self.ui.removeItem(item)

        if is_nan(x_position):
            return

        if not is_float(x_position):
            return
        new_item = pg.InfiniteLine(
            pos=x_position,
            movable=False,
            pen=pen,
            labelOpts={"position": 0.9},
            label=label,
        )
        self.ui.addItem(new_item)

        return new_item



import json
import os


def make_value_json_friendly(value):
    return str(value)


def load_json(json_file_name):
    if not os.path.exists(json_file_name):
        return None

    with open(json_file_name) as json_file:
        data = json.load(json_file)

    return data


def load_json_string(json_file_name):
    if not os.path.exists(json_file_name):
        return None

    json_string = load_json(json_file_name)
    dict = json.loads(json_string)

    return dict


def save_json(json_file_name, json_dictionary=None):
    with open(json_file_name, "w") as outfile:
        json.dump(json_dictionary, outfile)



#!/usr/bin/env python
"""
Get values from the GUI
"""

import numpy as np

from ibeatles.step6 import ParametersToDisplay


class Get:
    def __init__(self, parent=None):
        self.parent = parent

    def active_d0(self):
        if self.parent.ui.d0_value.isChecked():
            return float(self.parent.ui.d0_value.text())
        else:
            return float(self.parent.ui.d0_user_value.text())

    def parameter_to_display(self):
        if self.parent.ui.display_d_radioButton.isChecked():
            return ParametersToDisplay.d
        elif self.parent.ui.display_strain_mapping_radioButton.isChecked():
            return ParametersToDisplay.strain_mapping
        else:
            raise NotImplementedError("Parameters to display not implemented!")

    def strain_mapping(self):
        d_array = self.parent.d_array
        d0 = self.active_d0()
        strain_mapping = (d_array - d0) / d0

        if self.parent.min_max["strain_mapping"] is None:
            self.parent.min_max["strain_mapping"] = {
                "min": np.nanmin(strain_mapping),
                "max": np.nanmax(strain_mapping),
            }
            self.parent.min_max["strain_mapping"] = {
                "global_min": np.nanmin(strain_mapping),
                "global_max": np.nanmax(strain_mapping),
            }

        return strain_mapping

    def compact_strain_mapping(self):
        d_array = self.parent.compact_d_array
        d0 = self.active_d0()
        strain_mapping = (d_array - d0) / d0

        if self.parent.min_max["strain_mapping"] is None:
            self.parent.min_max["strain_mapping"] = {
                "min": np.nanmin(strain_mapping),
                "max": np.nanmax(strain_mapping),
            }
            self.parent.min_max["strain_mapping"] = {
                "global_min": np.nanmin(strain_mapping),
                "global_max": np.nanmax(strain_mapping),
            }

        return strain_mapping

    def d_array(self):
        return self.parent.d_array

    def integrated_image(self):
        return self.parent.integrated_image

    def strain_mapping_dictionary(self):
        d_dict = self.parent.d_dict
        strain_mapping_dict = {}
        for _row in d_dict.keys():
            d0 = self.active_d0()
            d = d_dict[_row]["val"]
            d_error = d_dict[_row]["err"]
            strain_mapping = (d - d0) / d0
            strain_mapping_err = d_error + np.sqrt(d0)

            strain_mapping_dict[_row] = {
                "val": strain_mapping,
                "err": strain_mapping_err,
            }

        return strain_mapping_dict

    def interpolation_method(self):
        return self.parent.ui.interpolation_comboBox.currentText()

    def cmap(self):
        return self.parent.ui.cmap_comboBox.currentText()

    def material_name(self):
        return str(self.parent.ui.material_name.text())

    def hkl_value(self):
        return str(self.parent.ui.hkl_value.text())



import os

root = os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
refresh_image = os.path.join(root, "icons/refresh.png")
settings_image = os.path.join(root, "icons/plotSettings.png")
combine_image = os.path.join(root, "icons/combine.png")
bin_image = os.path.join(root, "icons/bin.png")
auto_image = os.path.join(root, "icons/auto.png")
manual_image = os.path.join(root, "icons/manual.png")
more_infos_image = os.path.join(root, "icons/more_infos.png")
stats_table_image = os.path.join(root, "icons/stats_table.png")
stats_plot_image = os.path.join(root, "icons/stats_plot.png")

ANGSTROMS = "\u212b"
LAMBDA = "\u03bb"
MICRO = "\u00b5"
SUB_0 = "\u2080"
DELTA = "\u0394"


class SessionKeys:
    list_folders_status = "list_folders_status"
    list_folders = "list_folders"
    top_folder = "top_folder"

    combine_algorithm = "combine_algorithm"
    combine_roi = "combine_roi"
    combine_roi_item_id = "combine_roi_item_id"
    combine_image_view = "combine_image_view"

    # data folder
    folder = "folder"
    data = "data"
    list_files = "list_files"
    nbr_files = "nbr_files"
    use = "use"



import copy
import os
import time

import numpy as np
from astropy.io import fits
from PIL import Image


class ImageHandler(object):
    metadata = {}
    data_type = "tiff"
    data = []  # numpy array of image
    metadata = {}  # metadata dictionary
    filename = ""

    def __init__(self, parent=None, filename=None):
        self.data = []
        self.parent = parent

        # only first file loaded for now
        self.filename = filename
        self.retrieve_image_type()

    def retrieve_image_type(self):
        _file_0 = self.filename
        [_, file_extension] = os.path.splitext(_file_0)
        if (file_extension == ".tiff") or (file_extension == ".tif"):
            self.data_type = "tiff"
        elif file_extension == ".fits":
            self.data_type = "fits"
        else:
            raise ValueError("File Format not Supported!")

    def get_data(self):
        if self.filename == "":
            return []

        # if data not loaded yet
        if self.data == []:
            # only load first selected data
            # _file = self.filename
            if self.data_type == "tiff":
                self.get_tiff_data()
            elif self.data_type == "fits":
                self.get_fits_data()

            self.cleanup_data()

        return self.data

    def cleanup_data(self):
        _data = self.data

        where_are_nan = np.isnan(_data)
        _data[where_are_nan] = 0

        where_are_inf = np.isinf(_data)
        _data[where_are_inf] = 0
        self.data = _data

    def get_metadata(self, selected_infos_dict={}):
        if self.data == []:
            self.get_data()

        if self.data_type == "tiff":
            self.get_tiff_metadata(selected_infos_dict)
        elif self.data_type == "fits":
            self.get_fits_metadata(selected_infos_dict)

        return self.metadata

    def get_tiff_data(self):
        filename = self.filename
        _o_image = Image.open(filename)

        # metadata dict
        try:
            metadata = _o_image.tag_v2.as_dict()
        except AttributeError:
            metadata = None
        self.metadata = metadata

        # image
        data = np.array(_o_image)
        self.data = data

        _o_image.close()

    def get_fits_data(self):
        filename = self.filename
        self.data = copy.deepcopy(fits.getdata(filename))
        self.metadata = {}

    def get_tiff_metadata(self, selected_infos):
        _metadata = self.metadata

        # acquisition time
        try:  # new format
            acquisition_time_raw = _metadata[65000][0]
        except KeyError:
            acquisition_time_raw = _metadata[279][0]
        acquisition_time = time.ctime(acquisition_time_raw)
        selected_infos["acquisition_time"]["value"] = acquisition_time

        # acquisition duration
        try:
            acquisition_duration_raw = _metadata[65021][0]
            [_, value] = acquisition_duration_raw.split(":")
        except KeyError:
            value = "N/A"
        selected_infos["acquisition_duration"]["value"] = value

        # image size
        try:
            sizeX_raw = _metadata[65028][0]
            [_, valueX] = sizeX_raw.split(":")
        except KeyError:
            valueX = _metadata[256]

        try:
            sizeY_raw = _metadata[65029][0]
            [_, valueY] = sizeY_raw.split(":")
        except KeyError:
            valueY = _metadata[257]
        image_size = "{} x {}".format(valueX, valueY)
        selected_infos["image_size"]["value"] = image_size

        # image type
        bits = _metadata[258][0]
        selected_infos["image_type"]["value"] = "{} bits".format(bits)

        # min counts
        min_value = np.min(self.data)
        selected_infos["min_counts"]["value"] = "{}".format(min_value)

        # max counts
        max_value = np.max(self.data)
        selected_infos["max_counts"]["value"] = "{}".format(max_value)

        self.metadata = selected_infos

    def get_fits_metadata(self, selected_infos):
        _metadata = self.metadata
        _filename = self.filename

        try:
            # acquisition time
            try:  # new format
                acquisition_time = _metadata["DATE"]
            except KeyError:
                acquisition_time = time.ctime(os.path.getmtime(_filename))
            selected_infos["acquisition_duration"]["value"] = acquisition_time

            # acquisition duration
            try:
                acquisition_duration = _metadata["EXPOSURE"]
            except KeyError:
                acquisition_duration = _metadata["TiMEBIN"]
            selected_infos["acquisition_time"]["value"] = acquisition_duration

            # image size
            valueX = _metadata["NAXIS1"]
            valueY = _metadata["NAXIS2"]
            image_size = "{} x {}".format(valueX, valueY)
            selected_infos["image_size"]["value"] = image_size

            # image type
            bits = _metadata["BITPIX"]
            selected_infos["image_type"]["value"] = "{} bits".format(bits)

            # min counts
            min_value = np.min(self.data)
            selected_infos["min_counts"]["value"] = "{}".format(min_value)

            # max counts
            max_value = np.max(self.data)
            selected_infos["max_counts"]["value"] = "{}".format(max_value)

        except KeyError:
            selected_infos["acquisition_duration"]["value"] = "N/A"
            selected_infos["acquisition_time"]["value"] = "N/A"
            selected_infos["image_size"]["value"] = "N/A"
            selected_infos["image_type"]["value"] = "N/A"
            selected_infos["min_counts"]["value"] = "N/A"
            selected_infos["max_counts"]["value"] = "N/A"

        self.metadata = selected_infos



#!/usr/bin/env python
"""
Event Handler (step 1)
"""

import os

from loguru import logger

from ibeatles import DataType, Material
from ibeatles.all_steps.event_handler import EventHandler as TopEventHandler
from ibeatles.step1.data_handler import DataHandler
from ibeatles.step1.gui_handler import Step1GuiHandler
from ibeatles.step1.plot import Step1Plot
from ibeatles.step2.initialization import Initialization as Step2Initialization
from ibeatles.utilities.bragg_edge_element_handler import BraggEdgeElementHandler
from ibeatles.utilities.retrieve_data_infos import RetrieveGeneralDataInfos


class EventHandler(TopEventHandler):
    def import_button_clicked(self):
        logger.info(f"{self.data_type} import button clicked")

        self.parent.loading_flag = True
        o_load = DataHandler(parent=self.parent, data_type=self.data_type)
        _folder = o_load.select_folder()
        state = o_load.import_files_from_folder(
            folder=_folder,
            extension=None,
        )

        if state:
            self.import_button_clicked_step2(folder=_folder)
        else:
            logger.info("Import button clicked ... operation canceled!")

    def import_button_clicked_step2(self, folder):
        o_load = DataHandler(parent=self.parent, data_type=self.data_type)
        o_load.import_time_spectra()
        self.parent.select_load_data_row(data_type=self.data_type, row=0)
        self.parent.retrieve_general_infos(data_type=self.data_type)
        self.parent.retrieve_general_data_infos(data_type=self.data_type)
        BraggEdgeElementHandler(parent=self.parent)
        o_plot = Step1Plot(parent=self.parent, data_type=self.data_type)
        o_plot.initialize_default_roi()
        o_plot.display_bragg_edge(mouse_selection=False)
        o_gui = Step1GuiHandler(parent=self.parent, data_type=self.data_type)
        o_gui.check_time_spectra_widgets()
        o_gui.check_step1_widgets()
        # self.parent.check_files_error()
        o_step2_gui = Step2Initialization(parent=self.parent)
        o_step2_gui.roi()
        self.update_default_path(folder=folder)
        # activate or not infos button
        self.parent.infos_window_update(data_type=self.data_type)

        if self.data_type == DataType.sample:
            self.parent.data_metadata["normalization"]["data"] = []

        self.parent.ui.sample_ob_splitter.setSizes([20, 450])

    def sample_list_selection_changed(self):
        if not self.parent.loading_flag:
            o_retrieve_data_infos = RetrieveGeneralDataInfos(parent=self.parent, data_type=DataType.sample)
            o_retrieve_data_infos.update()
            self.parent.roi_image_view_changed(mouse_selection=False)
        else:
            self.parent.loading_flag = False

    def update_default_path(self, folder="./"):
        parent_folder = os.path.abspath(folder)
        if self.data_type == DataType.sample:
            self.parent.default_path[DataType.ob] = parent_folder
            self.parent.default_path[DataType.normalization] = parent_folder
        elif self.data_type == DataType.ob:
            self.parent.default_path[DataType.normalization] = parent_folder
        elif self.data_type == DataType.normalization:
            self.parent.default_path[DataType.normalized] = parent_folder

    def check_status_of_material_widgets(self):
        """
        this check if the lattice and crystal structure widgets can be displayed in the load data and normalized
        tab and changed the visibility of those widgets accordingly

        True if the selected element is from the default list, or if user used method1 when adding a new material
        False otherwise
        """
        element = self.parent.ui.list_of_elements.currentText()
        if element in self.parent.user_defined_bragg_edge_list.keys():
            if (
                self.parent.user_defined_bragg_edge_list[element][Material.method_used]
                == Material.via_lattice_and_crystal_structure
            ):
                self.parent.ui.crystal_structure_groupBox.setVisible(True)
                self.parent.ui.lattice_groupBox.setVisible(True)
            else:
                self.parent.ui.crystal_structure_groupBox.setVisible(False)
                self.parent.ui.lattice_groupBox.setVisible(False)
        else:
            self.parent.ui.crystal_structure_groupBox.setVisible(True)
            self.parent.ui.lattice_groupBox.setVisible(True)



#!/usr/bin/env python
"""
Fitting tab
"""

import numpy as np
from loguru import logger
from qtpy import QtCore
from qtpy.QtWidgets import QApplication, QMainWindow

from ibeatles import DataType, load_ui
from ibeatles.fitting import FittingTabSelected
from ibeatles.fitting.display import Display as FittingDisplay
from ibeatles.fitting.event_handler import EventHandler
from ibeatles.fitting.export import Export
from ibeatles.fitting.filling_table_handler import FillingTableHandler
from ibeatles.fitting.fitting_handler import FittingHandler
from ibeatles.fitting.get import Get
from ibeatles.fitting.initialization import Initialization
from ibeatles.fitting.kropff import RightClickTableMenu
from ibeatles.fitting.kropff import SessionSubKeys as KropffSessionSubKeys
from ibeatles.fitting.kropff.display import Display as KropffDisplay
from ibeatles.fitting.kropff.event_handler import EventHandler as KropffHandler
from ibeatles.fitting.kropff.export import Export as KropffExport
from ibeatles.fitting.kropff.kropff_automatic_settings_launcher import (
    KropffAutomaticSettingsLauncher,
)
from ibeatles.fitting.kropff.kropff_good_fit_settings_launcher import (
    KropffGoodFitSettingsLauncher,
)
from ibeatles.fitting.kropff.kropff_lambda_hkl_settings import (
    KropffLambdaHKLSettings,
)
from ibeatles.fitting.march_dollase.create_fitting_story_launcher import (
    CreateFittingStoryLauncher,
)
from ibeatles.fitting.march_dollase.event_handler import (
    EventHandler as MarchDollaseEventHandler,
)
from ibeatles.fitting.march_dollase.fitting_initialization_handler import (
    FittingInitializationHandler,
)
from ibeatles.fitting.selected_bin_handler import SelectedBinsHandler
from ibeatles.fitting.value_table_handler import ValueTableHandler
from ibeatles.step6.strain_mapping_launcher import StrainMappingLauncher
from ibeatles.table_dictionary.table_dictionary_handler import (
    TableDictionaryHandler,
)


class FittingLauncher:
    def __init__(self, parent=None):
        self.parent = parent

        if self.parent.fitting_ui is None:
            fitting_window = FittingWindow(parent=parent)
            fitting_window.show()
            self.parent.fitting_ui = fitting_window
            o_fitting = FittingHandler(grand_parent=self.parent, parent=self.parent.fitting_ui)
            o_fitting.display_image()
            o_fitting.display_roi()
            o_fitting.fill_table()
            try:
                fitting_window.record_all_xaxis_and_yaxis()
            except ValueError:
                pass
            fitting_window.bragg_edge_linear_region_changed(full_reset_of_fitting_table=False)
            fitting_window.kropff_check_widgets_helper()
            fitting_window.filling_kropff_table()
            fitting_window.update_locked_and_rejected_rows_in_bragg_peak_table()
            fitting_window.kropff_high_tof_table_selection_changed()
            # fitting_window.kropff_bragg_peak_auto_lock_rows_clicked()
            fitting_window.kropff_high_low_bragg_peak_tabs_changed(0)
            fitting_window.update_summary_table()

        else:
            self.parent.fitting_ui.setFocus()
            self.parent.fitting_ui.activateWindow()


class FittingWindow(QMainWindow):
    fitting_lr = None
    is_ready_to_fit = False

    lambda_0_item_in_bragg_edge_plot = None
    lambda_0_item_in_kropff_fitting_plot = None
    lambda_calculated_item_in_bragg_edge_plot = None
    lambda_calculated_item_in_kropff_fitting_plot = None

    data: list = []
    image_size = None  # [height, width]
    # there_is_a_roi = False
    bragg_edge_active_button_status = True  # to make sure active/lock button worked correctly

    list_bins_selected_item: list = []
    list_bins_locked_item: list = []

    image_view = None  # top left view
    image_view_scene = None  # scene of top left view
    image_view_item = None  # item of the top left view
    image_view_proxy = None  # proxy used when mouse moved in top left view

    bragg_edge_plot = None
    line_view = None

    line_view_fitting = None  # roi selected in binning window
    all_bins_button = None
    indi_bins_button = None

    header_value_tables_match = {
        0: [0],
        1: [1],
        2: [2],
        3: [3],
        4: [4],
        5: [5, 6],
        6: [7, 8],
        7: [9, 10],
        8: [11, 12],
        9: [13, 14],
        10: [15, 16],
        11: [17, 18],
        12: [19, 20],
    }

    para_cell_width = 130
    header_table_columns_width = [
        30,
        30,
        50,
        50,
        100,
        para_cell_width,
        para_cell_width,
        para_cell_width,
        para_cell_width,
        para_cell_width,
        para_cell_width,
        para_cell_width,
        para_cell_width,
        para_cell_width,
    ]
    fitting_table_columns_width = [
        header_table_columns_width[0],
        header_table_columns_width[1],
        header_table_columns_width[2],
        header_table_columns_width[3],
        header_table_columns_width[4],
        int(header_table_columns_width[5] / 2),
        int(header_table_columns_width[5] / 2),
        int(header_table_columns_width[6] / 2),
        int(header_table_columns_width[6] / 2),
        int(header_table_columns_width[7] / 2),
        int(header_table_columns_width[7] / 2),
        int(header_table_columns_width[8] / 2),
        int(header_table_columns_width[8] / 2),
        int(header_table_columns_width[9] / 2),
        int(header_table_columns_width[9] / 2),
        int(header_table_columns_width[10] / 2),
        int(header_table_columns_width[10] / 2),
        int(header_table_columns_width[11] / 2),
        int(header_table_columns_width[11] / 2),
        int(header_table_columns_width[12] / 2),
        int(header_table_columns_width[12] / 2),
    ]

    # status of alpha and sigma initialization
    sigma_alpha_initialized = False
    initialization_table = {
        "d_spacing": np.nan,
        "alpha": np.nan,
        "sigma": np.nan,
        "a1": np.nan,
        "a2": np.nan,
        "a5": np.nan,
        "a6": np.nan,
    }

    bragg_edge_data = {"x_axis": [], "y_axis": []}

    kropff_automatic_threshold_finder_algorithm = None
    kropff_threshold_current_item = None

    # kropff_bragg_peak_good_fit_conditions = {'l_hkl_error': {'state': True,
    #                                                          'value': 0.01},
    #                                          't_error'    : {'state': True,
    #                                                          'value': 0.01},
    #                                          'sigma_error': {'state': True,
    #                                                          'value': 0.01},
    #                                          }
    kropff_bragg_peak_good_fit_conditions = None

    # kropff_lambda_settings = {'state': 'fix',
    #                            'fix'  : 5e-8,
    #                            'range': [1e-8, 1e-7, 1e-8],
    #                            }
    kropff_lambda_settings = None

    # kropff_bragg_peak_row_rejections_conditions = {'l_hkl': {'less_than': {'state': True,
    #                                                                        'value': 0,},
    #                                                          'more_than': {'state': True,
    #                                                                        'value': 10,},
    #                                                          },
    #                                                }
    kropff_bragg_peak_row_rejections_conditions = None

    kropff_bragg_table_right_click_menu = {
        RightClickTableMenu.replace_values: {"ui": None, "state": False},
        RightClickTableMenu.display_fitting_parameters: {"ui": None, "state": False},
    }

    def __init__(self, parent=None):
        logger.info("Launching fitting tab!")

        self.parent = parent
        QMainWindow.__init__(self, parent=parent)
        self.ui = load_ui("ui_fittingWindow.ui", baseinstance=self)
        self.setWindowTitle("5. Fitting")

        o_init = Initialization(parent=self, grand_parent=self.parent)
        o_init.run_all()

        self.check_status_widgets()
        self.parent.session_dict[DataType.fitting]["ui_accessed"] = True

        x_axis = self.parent.normalized_lambda_bragg_edge_x_axis
        self.bragg_edge_data["x_axis"] = x_axis
        self.kropff_bragg_peak_good_fit_conditions = self.parent.session_dict[DataType.fitting][
            FittingTabSelected.kropff
        ][KropffSessionSubKeys.kropff_bragg_peak_good_fit_conditions]
        self.kropff_lambda_settings = self.parent.session_dict[DataType.fitting][FittingTabSelected.kropff][
            KropffSessionSubKeys.kropff_lambda_settings
        ]
        self.kropff_bragg_peak_row_rejections_conditions = self.parent.session_dict[DataType.fitting][
            FittingTabSelected.kropff
        ][KropffSessionSubKeys.bragg_peak_row_rejections_conditions]

    # MENU
    def action_strain_mapping_clicked(self):
        StrainMappingLauncher(parent=self.parent, fitting_parent=self)

    def action_configuration_for_cli_clicked(self):
        self.parent.save_session()
        o_export = Export(parent=self, grand_parent=self.parent)
        output_folder = o_export.select_output_folder()
        o_export.config_for_cli(output_folder=output_folder)

    def menu_export_table_as_ascii_clicked(self):
        o_export = KropffExport(parent=self, grand_parent=self.parent)
        o_export.ascii()
        self.ui.activateWindow()
        self.ui.setFocus()

    def menu_export_table_as_json_clicked(self):
        o_export = KropffExport(parent=self, grand_parent=self.parent)
        o_export.json()
        self.ui.activateWindow()
        self.ui.setFocus()

    def re_fill_table(self):
        o_fitting = FittingHandler(parent=self, grand_parent=self.parent)
        o_fitting.fill_table()

    def fitting_main_tab_widget_changed(self, index_tab=-1):
        pass
        # if index_tab == -1:
        #     index_tab = self.ui.tabWidget.currentIndex()
        #
        # o_fitting = FittingHandler(grand_parent=self.parent, parent=self)
        # o_fitting.display_locked_active_bins()
        # if index_tab == 1:
        #     self.bragg_edge_linear_region_changed(full_reset_of_fitting_table=False)
        #     o_event = KropffDisplay(parent=self, grand_parent=self.parent)
        #     o_event.display_bragg_peak_threshold()
        #
        # o_fitting_window_event = EventHandler(parent=self, grand_parent=self.parent)
        # o_fitting_window_event.check_widgets()

    # general fitting events

    def mouse_moved_in_image_view(self):
        self.image_view.setFocus(True)

    def hkl_list_changed(self, hkl):
        o_event = EventHandler(parent=self, grand_parent=self.parent)
        o_event.hkl_list_changed(hkl)

        o_kropff_display = KropffDisplay(parent=self, grand_parent=self.parent)
        o_kropff_display.display_lambda_0()

        o_fitting_display = FittingDisplay(parent=self, grand_parent=self.parent)
        o_fitting_display.display_lambda_0()

    def slider_changed(self):
        o_fitting_handler = FittingHandler(parent=self, grand_parent=self.parent)
        o_fitting_handler.display_roi()

    def update_bragg_edge_plot(self, update_selection=True):
        o_bin_handler = SelectedBinsHandler(parent=self, grand_parent=self.parent)
        o_bin_handler.update_bragg_edge_plot()
        if update_selection:
            self.bragg_edge_linear_region_changing()
        self.check_state_of_step3_button()

    def bragg_edge_linear_region_changing(self):
        self.is_ready_to_fit = False
        o_event = EventHandler(parent=self, grand_parent=self.parent)
        o_event.bragg_edge_region_changed()
        self.check_status_widgets()

    def bragg_edge_linear_region_changed(self, full_reset_of_fitting_table=True):
        o_table = TableDictionaryHandler(parent=self, grand_parent=self.parent)
        o_table.clear_y_axis_and_x_axis_from_kropff_table_dictionary()

        self.is_ready_to_fit = False
        o_event = EventHandler(parent=self, grand_parent=self.parent)
        o_event.bragg_edge_region_changed()
        self.check_status_widgets()

        o_get = Get(parent=self, grand_parent=self.parent)
        main_tab_selected = o_get.main_tab_selected()

        if main_tab_selected == FittingTabSelected.kropff:
            # we need to reset all kropff fitting parameters and plot
            if full_reset_of_fitting_table:
                o_kropff_event = KropffHandler(parent=self, grand_parent=self.parent)
                o_kropff_event.reset_fitting_parameters()

            self.kropff_check_widgets_helper()
            o_table = FillingTableHandler(parent=self, grand_parent=self.parent)
            o_table.fill_kropff_table()

            QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
            QApplication.processEvents()

            self.update_kropff_fitting_plots()

            o_event.automatically_select_best_lambda_0_for_that_range()

            o_kropff_display = KropffDisplay(parent=self, grand_parent=self.parent)
            o_kropff_display.display_lambda_0()
            o_kropff_display.display_lambda_calculated()

            o_fitting_display = FittingDisplay(parent=self, grand_parent=self.parent)
            o_fitting_display.display_lambda_0()

        QApplication.restoreOverrideCursor()
        QApplication.processEvents()

    def min_or_max_lambda_manually_changed(self):
        o_event = EventHandler(parent=self, grand_parent=self.parent)
        o_event.min_or_max_lambda_manually_changed()

    def create_fitting_story_checked(self):
        CreateFittingStoryLauncher(parent=self, grand_parent=self.parent)

    def automatic_hkl0_selection_clicked(self):
        o_event = EventHandler(parent=self, grand_parent=self.parent)
        o_event.automatic_hkl0_selection_clicked()
        o_event.automatically_select_best_lambda_0_for_that_range()

        o_kropff_display = KropffDisplay(parent=self, grand_parent=self.parent)
        o_kropff_display.display_lambda_0()

        o_fitting_display = FittingDisplay(parent=self, grand_parent=self.parent)
        o_fitting_display.display_lambda_0()

    # March-Dollase

    def column_value_table_clicked(self, column):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.column_value_table_clicked(column)

    def column_header_table_clicked(self, column):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.column_header_table_clicked(column)

    def resizing_header_table(self, index_column, old_size, new_size):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.resizing_header_table(index_column, new_size)

    def resizing_value_table(self, index_column, old_size, new_size):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.resizing_value_table(index_column, new_size)

    def active_button_pressed(self):
        self.parent.display_active_row_flag = True
        self.update_bragg_edge_plot()

    def lock_button_pressed(self):
        self.parent.display_active_row_flag = False
        self.update_bragg_edge_plot()

    def check_status_widgets(self):
        self.check_state_of_step3_button()
        self.check_state_of_step4_button()

    def check_state_of_step3_button(self):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.check_state_of_step3_button()

    def check_state_of_step4_button(self):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.check_state_of_step4_button()

    def active_button_state_changed(self, status, row_clicked):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.active_button_state_changed(row_clicked)

    def mirror_state_of_widgets(self, column=2, row_clicked=0):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.mirror_state_of_widgets(column=column, row_clicked=row_clicked)

    def lock_button_state_changed(self, status, row_clicked):
        o_event = MarchDollaseEventHandler(parent=self, grand_parent=self.parent)
        o_event.lock_button_state_changed(status, row_clicked)

    def value_table_right_click(self, position):
        o_table_handler = ValueTableHandler(grand_parent=self.parent, parent=self)
        o_table_handler.right_click(position=position)

    def check_advanced_table_status(self):
        self.is_ready_to_fit = False
        button_status = self.ui.advanced_table_checkBox.isChecked()
        self.advanced_table_clicked(button_status)
        self.check_status_widgets()

    def advanced_table_clicked(self, status):
        self.is_ready_to_fit = False
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        o_table_handler = FillingTableHandler(grand_parent=self.parent, parent=self)
        o_table_handler.set_mode(advanced_mode=status)
        self.check_status_widgets()
        QApplication.restoreOverrideCursor()

    def update_table(self):
        o_filling_table = FillingTableHandler(grand_parent=self.parent, parent=self)
        self.parent.fitting_ui.ui.value_table.blockSignals(True)
        o_filling_table.fill_table()
        self.parent.fitting_ui.ui.value_table.blockSignals(False)

    def initialize_all_parameters_button_clicked(self):
        o_initialization = FittingInitializationHandler(parent=self, grand_parent=self.parent)
        o_initialization.run()

    def initialize_all_parameters_step2(self):
        o_initialization = FittingInitializationHandler(parent=self, grand_parent=self.parent)
        o_initialization.finished_up_initialization()

        # activate or not step4 (yes if we were able to initialize correctly all variables)
        self.ui.step4_button.setEnabled(o_initialization.all_variables_initialized)
        self.ui.step4_label.setEnabled(o_initialization.all_variables_initialized)

        self.update_bragg_edge_plot()

    # kropff

    def mouse_clicked_in_top_left_image_view(self, mouse_click_event):
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        QApplication.processEvents()
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.mouse_clicked_in_top_left_image_view(mouse_click_event=mouse_click_event)
        QApplication.restoreOverrideCursor()
        QApplication.processEvents()

    def mouse_moved_in_top_left_image_view(self, evt):
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.mouse_moved_in_top_left_image_view(evt=evt)

    def filling_kropff_table(self):
        o_table = FillingTableHandler(parent=self, grand_parent=self.parent)
        o_table.fill_kropff_table()

    def kropff_check_widgets_helper(self):
        """highlight in green the next button to use"""
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.check_widgets_helper()

    def record_all_xaxis_and_yaxis(self):
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.record_all_xaxis_and_yaxis()

    def kropff_high_low_bragg_peak_tabs_changed(self, tab_index):
        self.update_kropff_fitting_plots()
        self.update_selected_bins_plot()
        o_display = KropffDisplay(parent=self, grand_parent=self.parent)
        o_display.display_bragg_peak_threshold()
        # o_display.update_fitting_parameters_matplotlib()

    def kropff_automatic_bragg_peak_threshold_finder_clicked(self):
        self.bragg_edge_linear_region_changed(full_reset_of_fitting_table=True)
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.reset_fitting_parameters()
        o_table = FillingTableHandler(parent=self, grand_parent=self.parent)
        o_table.fill_kropff_table()
        o_event.kropff_automatic_bragg_peak_threshold_finder_clicked()
        self.kropff_check_widgets_helper()

    def kropff_automatic_fit_clicked(self):
        self.ui.kropff_tabWidget.setCurrentIndex(3)
        self.kropff_automatic_bragg_peak_threshold_finder_clicked()
        self.kropff_fit_all_regions()

    def kropff_automatic_bragg_peak_threshold_finder_settings_clicked(self):
        o_kropff = KropffAutomaticSettingsLauncher(parent=self, grand_parent=self.parent)
        o_kropff.show()

    def kropff_parameters_changed(self):
        o_kropff = KropffHandler(parent=self, grand_parent=self.parent)
        o_kropff.parameters_changed()

    def update_selected_bins_plot(self):
        o_kropff = SelectedBinsHandler(parent=self, grand_parent=self.parent)
        o_kropff.update_bins_selected()
        o_kropff.update_bragg_edge_plot()

    def update_kropff_fitting_plots(self):
        o_kropff = KropffHandler(parent=self, grand_parent=self.parent)
        o_kropff.update_fitting_plots()

    def kropff_parameters_changed_with_string(self, string):
        self.kropff_parameters_changed()

    def kropff_high_tof_table_selection_changed(self):
        # kropff_table_dictionary = self.parent.kropff_table_dictionary
        self.update_bragg_edge_plot()
        self.update_kropff_fitting_plots()
        self.update_selected_bins_plot()
        o_event = KropffDisplay(parent=self, grand_parent=self.parent)
        o_event.display_bragg_peak_threshold()

    def kropff_low_tof_table_selection_changed(self):
        self.update_bragg_edge_plot()
        self.update_kropff_fitting_plots()
        self.update_selected_bins_plot()
        o_event = KropffDisplay(parent=self, grand_parent=self.parent)
        o_event.display_bragg_peak_threshold()

    def kropff_bragg_peak_table_selection_changed(self):
        self.update_selected_bins_plot()
        self.update_bragg_edge_plot()
        self.update_kropff_fitting_plots()
        o_event = KropffDisplay(parent=self, grand_parent=self.parent)
        o_event.display_bragg_peak_threshold()

    def kropff_bragg_edge_threshold_changed(self):
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.kropff_bragg_edge_threshold_changed()

    def kropff_fit_all_regions(self):
        self.ui.kropff_tabWidget.setCurrentIndex(3)
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.fit_regions()
        o_event.update_fitting_plots()
        o_event.check_how_many_fitting_are_locked()
        o_display = KropffDisplay(parent=self, grand_parent=self.parent)
        o_display.display_bragg_peak_threshold()
        o_table = FillingTableHandler(parent=self, grand_parent=self.parent)
        o_table.fill_kropff_table()
        o_display.update_fitting_parameters_matplotlib()
        self.update_locked_and_rejected_rows_in_bragg_peak_table()
        self.update_summary_table()
        # self.kropff_bragg_table_right_click_menu[RightClickTableMenu.replace_values]['state'] = True

    # fitting settings

    def kropff_initial_guess_lambda_hkl_fix_clicked(self):
        o_event = KropffHandler(parent=self)
        o_event.change_initial_guess_lambda_hkl_widgets()

    def kropff_initial_guess_lambda_hkl_range_clicked(self):
        o_event = KropffHandler(parent=self)
        o_event.change_initial_guess_lambda_hkl_widgets()

    def kropff_initial_guess_sigma_fix_clicked(self):
        o_event = KropffHandler(parent=self)
        o_event.change_initial_guess_sigma_widgets()

    def kropff_initial_guess_sigma_range_clicked(self):
        o_event = KropffHandler(parent=self)
        o_event.change_initial_guess_sigma_widgets()

    def kropff_initial_guess_tau_fix_clicked(self):
        o_event = KropffHandler(parent=self)
        o_event.change_initial_guess_tau_widgets()

    def kropff_initial_guess_tau_range_clicked(self):
        o_event = KropffHandler(parent=self)
        o_event.change_initial_guess_tau_widgets()

    def update_summary_table(self):
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.update_summary_table()

    def kropff_high_tof_graph_radioButton_changed(self):
        o_event = KropffDisplay(parent=self, grand_parent=self.parent)
        o_event.update_fitting_parameters_matplotlib()

    def kropff_low_tof_graph_radioButton_changed(self):
        o_event = KropffDisplay(parent=self, grand_parent=self.parent)
        o_event.update_fitting_parameters_matplotlib()

    def kropff_bragg_peak_graph_radioButton_changed(self):
        o_event = KropffDisplay(parent=self, grand_parent=self.parent)
        o_event.update_fitting_parameters_matplotlib()

    def kropff_bragg_peak_number_of_digits_changed(self):
        o_table = FillingTableHandler(parent=self, grand_parent=self.parent)
        o_table.fill_kropff_bragg_peak_table()

    def kropff_fit_bragg_peak_button_clicked(self):
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.fit_bragg_peak()
        o_event.check_how_many_fitting_are_locked()

    def kropff_bragg_peak_table_right_clicked(self, position):
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        o_event.bragg_peak_right_click()
        # o_event.check_how_many_fitting_are_locked()

    # def kropff_bragg_peak_auto_lock_rows_clicked(self):
    #     o_event = KropffHandler(parent=self, grand_parent=self.parent)
    #     o_event.bragg_peak_auto_lock_clicked()

    def kropff_bragg_peak_good_fit_settings_clicked(self):
        o_kropff = KropffGoodFitSettingsLauncher(parent=self)
        o_kropff.show()

    def update_locked_and_rejected_rows_in_bragg_peak_table(self):
        o_event = KropffHandler(parent=self, grand_parent=self.parent)
        # o_event.bragg_peak_auto_lock_clicked()
        o_event.check_how_many_fitting_are_locked()

    def kropff_bragg_peak_lambda_settings_clicked(self):
        o_lambda = KropffLambdaHKLSettings(parent=self, grand_parent=self.parent)
        o_lambda.show()

    # def kropff_fitting_parameters_viewer_editor_clicked(self):
    #     KropffFittingParametersViewerEditorLauncher(parent=self,
    #                                                 grand_parent=self.parent)

    # general settings

    def windows_settings(self):
        self.parent.session_dict[DataType.fitting]["ui"]["kropff_top_horizontal_splitter"] = (
            self.ui.kropff_top_horizontal_splitter.sizes()
        )
        self.parent.session_dict[DataType.fitting]["ui"]["splitter_2"] = self.ui.splitter_2.sizes()
        self.parent.session_dict[DataType.fitting]["ui"]["splitter_3"] = self.ui.splitter_3.sizes()
        self.parent.session_dict[DataType.fitting]["ui"]["splitter_4"] = self.ui.splitter_4.sizes()

    def save_all_parameters(self):
        self.kropff_parameters_changed()
        self.windows_settings()

    def closeEvent(self, event=None):
        self.save_all_parameters()
        if self.parent.advanced_selection_ui:
            self.parent.advanced_selection_ui.close()
        if self.parent.fitting_set_variables_ui:
            self.parent.fitting_set_variables_ui.close()
        self.parent.fitting_ui = None



#!/usr/bin/env python
"""
Load normalization tab
"""

from ibeatles import DataType
from ibeatles.session import SessionSubKeys
from ibeatles.step2.gui_handler import Step2GuiHandler
from ibeatles.step2.initialization import Initialization as Step2Initialization
from ibeatles.utilities.pyqrgraph import Pyqtgrah as PyqtgraphUtilities


class LoadNormalization:
    def __init__(self, parent=None):
        self.parent = parent
        self.session_dict = parent.session_dict
        self.data_type = DataType.normalization

    def roi(self):
        data_type = self.data_type
        session_dict = self.session_dict

        list_roi = session_dict[data_type][SessionSubKeys.roi]
        self.parent.list_roi[data_type] = list_roi

        o_step2 = Step2Initialization(parent=self.parent)
        o_step2.roi()

    def check_widgets(self):
        o_step2 = Step2GuiHandler(parent=self.parent)
        o_step2.check_run_normalization_button()

    def image_settings(self):
        data_type = self.data_type
        session_dict = self.session_dict

        self.parent.image_view_settings[data_type]["state"] = session_dict[data_type][SessionSubKeys.image_view_state]
        self.parent.image_view_settings[data_type]["histogram"] = session_dict[data_type][
            SessionSubKeys.image_view_histogram
        ]

        o_pyqt = PyqtgraphUtilities(
            parent=self.parent,
            image_view=self.parent.step2_ui["image_view"],
            data_type=data_type,
        )
        o_pyqt.set_state(session_dict[data_type][SessionSubKeys.image_view_state])
        o_pyqt.reload_histogram_level()
        histogram_level = session_dict[data_type][SessionSubKeys.image_view_histogram]
        o_pyqt.set_histogram_level(histogram_level=histogram_level)



import numpy as np
from qtpy import QtCore, QtGui
from qtpy.QtWidgets import QTableWidgetItem


class SetFittingVariablesHandler:
    colorscale_nbr_row = 15
    colorscale_cell_size = {"width": 75, "height": 30}

    def __init__(self, grand_parent=None):
        self.grand_parent = grand_parent

    def get_variable_selected(self):
        if self.grand_parent.fitting_set_variables_ui.ui.lambda_hkl_button.isChecked():
            return "lambda_hkl"
        elif self.grand_parent.fitting_set_variables_ui.ui.sigma_button.isChecked():
            return "sigma"
        elif self.grand_parent.fitting_set_variables_ui.ui.tau_button.isChecked():
            return "tau"

    def populate_table_with_variable(self, variable=None):
        if variable is None:
            variable = self.get_variable_selected()

        array_2d_values = self.create_array_of_variable(variable=variable)

        print(array_2d_values)

        # # retrieve min and max values
        # if np.isnan(array_2d_values).any():
        #     min_value = np.nan
        #     max_value = np.nan
        #     mid_point = np.nan
        # else:
        #     min_value = np.nanmin(array_2d_values)
        #     max_value = np.nanmax(array_2d_values)
        #     mid_point = np.mean([min_value, max_value])
        #
        # # define colorscale table
        # self.initialize_colorscale_table(min_value=min_value, max_value=max_value)
        #
        # [nbr_row, nbr_column] = np.shape(array_2d_values)
        # for _row in np.arange(nbr_row):
        #     for _col in np.arange(nbr_column):
        #         _value = array_2d_values[_row, _col]
        #         _color = self.get_color_for_this_value(min_value=min_value,
        #                                                max_value=max_value,
        #                                                value=_value)
        #         _item = QTableWidgetItem(str(_value))
        #         _item.setBackground(_color)
        #
        #         bin_index = _row + nbr_row * _col
        #         if self.is_bin_locked(bin_index=bin_index):
        #             _gradient = QtGui.QRadialGradient(10, 10, 10, 20, 20)
        #             _gradient.setColorAt(1, _color)
        #             if self.is_bin_fixed(bin_index=bin_index, variable_name=variable):
        #                 _gradient.setColorAt(0.5, QtGui.QColor(255, 255, 255))
        #             _gradient.setColorAt(0, QtGui.QColor(255, 0, 0, alpha=255))
        #             _item.setBackground(QtGui.QBrush(_gradient))
        #         elif self.is_bin_activated(bin_index=bin_index):
        #             _gradient = QtGui.QRadialGradient(10, 10, 10, 20, 20)
        #             _gradient.setColorAt(1, _color)
        #             if self.is_bin_fixed(bin_index=bin_index, variable_name=variable):
        #                 _gradient.setColorAt(0.5, QtGui.QColor(255, 255, 255))
        #             _gradient.setColorAt(0, QtGui.QColor(0, 0, 255, alpha=255))
        #             _item.setBackground(QtGui.QBrush(_gradient))
        #         else:
        #             if self.is_bin_fixed(bin_index=bin_index, variable_name=variable):
        #                 _gradient = QtGui.QRadialGradient(10, 10, 10, 20, 20)
        #                 _gradient.setColorAt(1, _color)
        #                 _gradient.setColorAt(0, QtGui.QColor(255, 255, 255))
        #                 _item.setBackground(QtGui.QBrush(_gradient))
        #
        #         if (_value > mid_point):
        #             _foreground_color = QtGui.QColor(255, 255, 255, alpha=255)
        #             _item.setTextColor(_foreground_color)
        #
        #         self.grand_parent.fitting_set_variables_ui.ui.variable_table.setItem(_row, _col, _item)

    def is_bin_fixed(self, bin_index=0, variable_name="d_spacing"):
        table_dictionary = self.grand_parent.march_table_dictionary
        return table_dictionary[str(bin_index)][variable_name]["fixed"]

    def is_bin_locked(self, bin_index=0):
        table_dictionary = self.grand_parent.march_table_dictionary
        return table_dictionary[str(bin_index)]["lock"]

    def is_bin_activated(self, bin_index=0):
        table_dictionary = self.grand_parent.march_table_dictionary
        return table_dictionary[str(bin_index)]["active"]

    def clear_colorscale_table(self):
        nbr_row = self.grand_parent.fitting_set_variables_ui.ui.colorscale_table.rowCount()
        for _row in np.arange(nbr_row):
            self.grand_parent.fitting_set_variables_ui.ui.colorscale_table.removeRow(0)

    def initialize_colorscale_table(self, min_value=0, max_value=1):
        self.clear_colorscale_table()

        nbr_row = self.colorscale_nbr_row
        step = (float(max_value) - float(min_value)) / (nbr_row - 1)
        mid_point = int(nbr_row / 2)
        _row = 0

        if min_value == max_value:
            nbr_row = 1

        if np.isnan(step):
            nbr_row = 1

        for _index in np.arange(nbr_row - 1, -1, -1):
            self.grand_parent.fitting_set_variables_ui.ui.colorscale_table.insertRow(_row)
            self.grand_parent.fitting_set_variables_ui.ui.colorscale_table.setRowHeight(
                _row, self.colorscale_cell_size["height"]
            )
            self.grand_parent.fitting_set_variables_ui.ui.colorscale_table.setColumnWidth(
                _row, self.colorscale_cell_size["width"]
            )
            if np.isnan(step):
                _value = np.nan
            else:
                _value = min_value + _index * step
            _color = self.get_color_for_this_value(min_value=min_value, max_value=max_value, value=_value)

            if np.isnan(_value):
                _item = QTableWidgetItem("nan")
            else:
                _item = QTableWidgetItem("{:04.2f}".format(_value))
            _item.setBackground(_color)
            _item.setTextAlignment(QtCore.Qt.AlignRight)
            if (_row < mid_point) and (nbr_row != 1):
                # font should be changed from black to white
                _foreground_color = QtGui.QColor(255, 255, 255, alpha=255)
                _item.setTextColor(_foreground_color)

            self.grand_parent.fitting_set_variables_ui.ui.colorscale_table.setItem(_row, 0, _item)
            _row += 1

    def get_color_for_this_value(self, min_value=0, max_value=1, value=0):
        if np.isnan(value):
            return QtGui.QColor(255, 255, 255, alpha=100)  # white
        elif max_value == min_value:
            return QtGui.QColor(250, 0, 0, alpha=255)  # red

        _ratio = 1 - (float(value) - float(min_value)) / (float(max_value) - float(min_value))
        return QtGui.QColor(0, _ratio * 255, 0, alpha=255)

    def create_array_of_variable(self, variable="d_spacing"):
        table_dictionary = self.kropf_table_dictionary

        _table_selection = self.grand_parent.fitting_selection
        nbr_column = _table_selection["nbr_column"]
        nbr_row = _table_selection["nbr_row"]

        _array = np.zeros((nbr_row, nbr_column), dtype=float)

        for _entry in table_dictionary.keys():
            row_index = table_dictionary[_entry]["row_index"]
            column_index = table_dictionary[_entry]["column_index"]
            value = table_dictionary[_entry][variable]["val"]
            _array[row_index, column_index] = value

        return _array

    def set_new_value_to_selected_bins(
        self, selection=[], variable_name="d_spacing", variable_value=0, table_nbr_row=0
    ):
        table_dictionary = self.grand_parent.march_table_dictionary
        nbr_row = table_nbr_row

        for _select in selection:
            _left_column = _select.leftColumn()
            _right_column = _select.rightColumn()
            _top_row = _select.topRow()
            _bottom_row = _select.bottomRow()
            for _row in np.arange(_top_row, _bottom_row + 1):
                for _col in np.arange(_left_column, _right_column + 1):
                    _index = str(_row + _col * nbr_row)
                    if not table_dictionary[_index]["lock"]:
                        table_dictionary[_index][variable_name]["val"] = float(variable_value)
                        table_dictionary[_index][variable_name]["err"] = np.nan
            self.grand_parent.fitting_set_variables_ui.ui.variable_table.setRangeSelected(_select, False)

        self.grand_parent.march_table_dictionary = table_dictionary
        self.populate_table_with_variable(variable=variable_name)



#!/usr/bin/env python
"""
KropffLambdaHKLSettings class for handling the settings of the Kropff Lambda HKL.
"""

from qtpy.QtWidgets import QDialog

from ibeatles import load_ui


class KropffLambdaHKLSettings(QDialog):
    ### REMOVE_ME

    fit_conditions: dict = {}

    def __init__(self, parent=None, grand_parent=None):
        self.parent = parent
        self.grand_parent = grand_parent
        super(QDialog, self).__init__(parent)
        self.ui = load_ui("ui_kropff_lambda_hkl_settings.ui", baseinstance=self)
        self.init_widgets()

    def init_widgets(self):
        self.ui.fix_lineEdit.setText(str(self.parent.kropff_lambda_settings["fix"]))
        from_value, to_value, step_value = self.parent.kropff_lambda_settings["range"]
        self.ui.from_lineEdit.setText(str(from_value))
        self.ui.to_lineEdit.setText(str(to_value))
        self.ui.step_lineEdit.setText(str(step_value))
        if self.parent.kropff_lambda_settings["state"] == "fix":
            self.ui.fix_radioButton.setChecked(True)
        else:
            self.ui.range_radioButton.setChecked(True)
        self.radio_button_changed()

    def done(self, value):
        self.ok_clicked()
        QDialog.done(self, value)

    def ok_clicked(self):
        if self.ui.fix_radioButton.isChecked():
            self.parent.kropff_lambda_settings["state"] = "fix"
        else:
            self.parent.kropff_lambda_settings["state"] = "range"

        try:
            fix_value = float(str(self.ui.fix_lineEdit.text()))
            from_value = float(str(self.ui.from_lineEdit.text()))
            to_value = float(str(self.ui.to_lineEdit.text()))
            step_value = float(str(self.ui.step_lineEdit.text()))
        except ValueError:
            self.init_widgets()
            return

        self.parent.kropff_lambda_settings["fix"] = fix_value
        self.parent.kropff_lambda_settings["range"] = [from_value, to_value, step_value]
        self.close()

    def radio_button_changed(self):
        fix_state = self.ui.fix_radioButton.isChecked()
        self.ui.fix_lineEdit.setEnabled(fix_state)
        self.ui.from_lineEdit.setEnabled(not fix_state)
        self.ui.to_label.setEnabled(not fix_state)
        self.ui.to_lineEdit.setEnabled(not fix_state)
        self.ui.step_label.setEnabled(not fix_state)
        self.ui.step_lineEdit.setEnabled(not fix_state)

    def check_ok_button(self):
        state1 = self.ui.lambda_hkl_checkBox.isChecked()
        state2 = self.ui.tau_checkBox.isChecked()
        state3 = self.ui.sigma_checkBox.isChecked()

        if (not state1) and (not state2) and (not state3):
            button_state = False
        else:
            button_state = True

        self.ui.ok_pushButton.setEnabled(button_state)



#!/usr/bin/env python
"""
Normalization
"""

import copy
import os
import shutil

import numpy as np
from loguru import logger
from NeuNorm.normalization import Normalization as NeuNormNormalization
from NeuNorm.roi import ROI
from qtpy.QtWidgets import QFileDialog

from ibeatles import DataType
from ibeatles.session import ReductionDimension, SessionKeys, SessionSubKeys
from ibeatles.step2.get import Get
from ibeatles.step2.reduction_settings_handler import ReductionSettingsHandler
from ibeatles.step2.reduction_tools import moving_average
from ibeatles.step2.roi_handler import Step2RoiHandler
from ibeatles.step3.event_handler import EventHandler
from ibeatles.utilities.file_handler import FileHandler
from ibeatles.utilities.status_message_config import (
    StatusMessageStatus,
    show_status_message,
)


class Normalization:
    coeff_array = 1  # ob / sample of ROI selected
    o_norm = None

    def __init__(self, parent=None):
        self.parent = parent

    def run_and_export(self):
        logger.info("Running and exporting normalization:")

        # ask for output folder location
        sample_folder = self.parent.data_metadata["sample"]["folder"]
        sample_name = os.path.basename(os.path.dirname(sample_folder))
        default_dir = os.path.dirname(os.path.dirname(sample_folder))
        output_folder = str(
            QFileDialog.getExistingDirectory(
                caption="Select Where the Normalized folder will be created...",
                directory=default_dir,
                options=QFileDialog.ShowDirsOnly,
            )
        )

        if not output_folder:
            logger.info(" No output folder selected, normalization stopped!")
            return False

        # save output folder to session
        self.parent.session_dict[DataType.normalized][SessionSubKeys.current_folder] = output_folder

        logger.info(f" output folder selected: {output_folder}")
        full_output_folder = os.path.join(output_folder, sample_name + "_normalized")
        try:
            full_output_folder = FileHandler.make_or_append_date_time_to_folder(full_output_folder)
        except OSError:
            logger.info(f"ERROR: folder permission error into this folder {full_output_folder}")
            show_status_message(
                parent=self.parent,
                message="You don't have write permission into this folder!",
                status=StatusMessageStatus.error,
                duration_s=10,
            )
            return False

        logger.info(f" full output folder will be: {full_output_folder}")

        o_norm = self.create_o_norm()

        if self.parent.session_dict[SessionKeys.reduction][SessionSubKeys.process_order] == "option1":
            # running moving average before running normalization
            o_norm = self.running_moving_average(o_norm=copy.deepcopy(o_norm))
            o_norm = self.running_normalization(o_norm=copy.deepcopy(o_norm))
        else:
            # running normalization then moving average
            o_norm = self.running_normalization(o_norm=copy.deepcopy(o_norm))
            o_norm = self.running_moving_average(o_norm=copy.deepcopy(o_norm))

        if not o_norm:
            logger.info("Normalization failed!")
            show_status_message(
                parent=self.parent,
                message="Normalization Failed (check logbook)!",
                status=StatusMessageStatus.error,
            )
            return False

        self.export_normalization(o_norm=o_norm, output_folder=full_output_folder)
        self.saving_normalization_parameters(o_norm=o_norm, output_folder=full_output_folder)
        self.moving_time_spectra_to_normalizaton_folder(output_folder=full_output_folder)

        # repopulate ui with normalized data
        o_step3 = EventHandler(parent=self.parent, data_type=DataType.normalized)
        o_step3.import_button_clicked_automatically(folder=full_output_folder)

        return True

    def create_o_norm(self):
        logger.info("Creating o_norm object (to prepare data normalization!")

        _data = self.parent.data_metadata["sample"]["data"]
        _ob = self.parent.data_metadata["ob"]["data"]

        show_status_message(
            parent=self.parent,
            message="Loading data ...",
            status=StatusMessageStatus.working,
        )
        o_norm = NeuNormNormalization()
        o_norm.load(data=_data)
        show_status_message(
            parent=self.parent,
            message="Loading data ... Done!",
            status=StatusMessageStatus.working,
            duration_s=5,
        )

        if _ob.any():
            show_status_message(
                parent=self.parent,
                message="Loading ob data ...",
                status=StatusMessageStatus.working,
            )
            o_norm.load(data=_ob, data_type=DataType.ob)
            show_status_message(
                parent=self.parent,
                message="Loading ob data ... Done!",
                status=StatusMessageStatus.working,
                duration_s=5,
            )

        return o_norm

    def export_normalization(self, o_norm=None, output_folder=None):
        show_status_message(
            parent=self.parent,
            message="Exporting normalized files ...",
            status=StatusMessageStatus.working,
        )
        o_norm.export(folder=output_folder)
        show_status_message(
            parent=self.parent,
            message="Exporting normalized files ... Done!",
            status=StatusMessageStatus.working,
            duration_s=5,
        )

    def moving_time_spectra_to_normalizaton_folder(self, output_folder=None):
        logger.info("Copying time spectra file from input folder to output folder.")
        time_spectra = self.parent.data_metadata["sample"]["time_spectra"]
        filename = time_spectra["filename"]
        folder = time_spectra["folder"]
        full_time_spectra = os.path.join(folder, filename)
        logger.info(f"-> time_spectra: {time_spectra}")
        logger.info(f"-> full_time_spectra: {full_time_spectra}")
        shutil.copy(full_time_spectra, output_folder)

    def saving_normalization_parameters(self, o_norm=None, output_folder=None):
        logger.info("Internally saving normalization parameters (data, folder, time_spectra)")
        self.parent.data_metadata[DataType.normalized]["data"] = np.array(o_norm.get_normalized_data())
        self.parent.data_metadata[DataType.normalized]["folder"] = output_folder
        self.parent.data_metadata[DataType.normalized]["time_spectra"] = copy.deepcopy(
            self.parent.data_metadata[DataType.sample]["time_spectra"]
        )

    def running_moving_average(self, o_norm=None):
        if o_norm is None:
            return None

        running_moving_average_settings = self.parent.session_dict[SessionKeys.reduction]
        if not running_moving_average_settings["activate"]:
            logger.info("Not running moving average! Option has been turned off")
            return o_norm

        show_status_message(
            parent=self.parent,
            message="Running moving average ...",
            status=StatusMessageStatus.working,
        )
        logger.info("Running moving average:")
        reduction_settings = self.parent.session_dict[SessionKeys.reduction]

        if reduction_settings["size"]["flag"] == "default":
            x = ReductionSettingsHandler.default_kernel_size["x"]
            y = ReductionSettingsHandler.default_kernel_size["y"]
            lda = ReductionSettingsHandler.default_kernel_size["l"]

        else:
            x = reduction_settings["size"]["x"]
            y = reduction_settings["size"]["y"]
            lda = reduction_settings["size"]["l"]

        kernel = [y, x]
        if reduction_settings["dimension"] == ReductionDimension.threed:
            kernel.append(lda)

        _data = np.array(o_norm.data[DataType.sample]["data"])  # lambda, x, y
        _data_transposed = _data.transpose(2, 1, 0)  # x, y, lambda

        o_get = Get(parent=self.parent)
        kernel_type = o_get.kernel_type()

        logger.info(f"-> kernel dimension: {reduction_settings['dimension']}")
        logger.info(f"-> kernel shape: {np.shape(kernel)}")
        logger.info(f"-> len(sample): {len(_data_transposed)}")
        logger.info(f"-> kernel: {kernel}")
        logger.info(f"-> kernel type: {kernel_type}")

        logger.info("-> Starting to run moving average with sample data")
        show_status_message(
            parent=self.parent,
            message="Moving average of sample data ...",
            status=StatusMessageStatus.working,
        )
        sample_data = moving_average(data=_data, kernel=kernel, kernel_type=kernel_type)
        logger.info("-> Done running moving average with sample data!")
        if sample_data is None:
            logger.info("Moving average failed!")
            show_status_message(
                parent=self.parent,
                message="Running moving average ... Failed",
                status=StatusMessageStatus.error,
            )
            return
        else:
            sample_data.transpose(2, 1, 0)  # lambda, x, y

        o_norm.data[DataType.sample]["data"] = sample_data
        show_status_message(
            parent=self.parent,
            message="Moving average of sample data ... Done!",
            status=StatusMessageStatus.working,
            duration_s=5,
        )

        _ob = np.array(o_norm.data[DataType.ob]["data"])
        if _ob is None:
            show_status_message(
                parent=self.parent,
                message="Moving average of ob data ...",
                status=StatusMessageStatus.ready,
            )
            logger.info("-> Starting to run moving average with ob data")
            ob_data = moving_average(data=_ob, kernel=kernel, kernel_type=kernel_type)
            logger.info("-> Done running moving average with ob data!")
            if ob_data:
                ob_data.transpose(2, 1, 0)  # lambda, x, y
            else:
                logger.info("Moving average failed!")
                show_status_message(
                    parent=self.parent,
                    message="Running moving average ... Failed",
                    status=StatusMessageStatus.error,
                )
                return

            o_norm.data[DataType.ob]["data"] = ob_data

            show_status_message(
                parent=self.parent,
                message="Moving average of ob data ... Done!",
                status=StatusMessageStatus.ready,
                duration_s=5,
            )

        return o_norm

    def running_normalization(self, o_norm=None):
        logger.info(" running normalization!")

        # if o_norm is None:
        #     _data = self.parent.data_metadata['sample']['data']
        #     _ob = self.parent.data_metadata['ob']['data']
        # else:
        #     _data = o_norm.data[DataType.sample]['data']
        #     _ob = o_norm.data[DataType.ob]['data']

        # check if roi selected or not
        o_roi_handler = Step2RoiHandler(parent=self.parent)
        try:  # to avoid valueError when row not fully filled
            list_roi_to_use = o_roi_handler.get_list_of_background_roi_to_use()
        except ValueError:
            logger.info(" Error raised when retrieving the background ROI!")
            return None

        logger.info(f" Background list of ROI: {list_roi_to_use}")

        if not o_norm.data["ob"]["data"]:
            # if just sample data
            return self.normalization_only_sample_data(o_norm, list_roi_to_use)
        else:
            # if ob
            return self.normalization_sample_and_ob_data(o_norm, list_roi_to_use)

    def normalization_only_sample_data(self, o_norm, list_roi):
        logger.info(" running normalization with only sample data ...")

        # show_status_message(parent=self.parent,
        #                     message="Loading data ...",
        #                     status=StatusMessageStatus.working)
        # o_norm = NeuNormNormalization()
        # o_norm.load(data=data)
        # show_status_message(parent=self.parent,
        #                     message="Loading data ... Done!",
        #                     status=StatusMessageStatus.working,
        #                     duration_s=5)

        list_roi_object = []
        for _roi in list_roi:
            o_roi = ROI(
                x0=int(_roi[0]),
                y0=int(_roi[1]),
                width=int(_roi[2]),
                height=int(_roi[3]),
            )
            list_roi_object.append(o_roi)

        show_status_message(
            parent=self.parent,
            message="Running normalization ...",
            status=StatusMessageStatus.working,
        )
        o_norm.normalization(roi=list_roi_object, use_only_sample=True)

        # self.o_norm = o_norm

        show_status_message(
            parent=self.parent,
            message="Running normalization ... Done!",
            status=StatusMessageStatus.working,
            duration_s=5,
        )

        logger.info(" running normalization with only sample data ... Done!")
        return o_norm

    def normalization_sample_and_ob_data(self, o_norm, list_roi):
        logger.info(" running normalization with sample and ob data ...")

        # # sample
        # show_status_message(parent=self.parent,
        #                     message="Loading sample data ...",
        #                     status=StatusMessageStatus.working)
        # o_norm = NeuNormNormalization()
        # o_norm.load(data=data)
        # show_status_message(parent=self.parent,
        #                     message="Loading sample data ... Done!",
        #                     status=StatusMessageStatus.working)
        #
        # # ob
        # show_status_message(parent=self.parent,
        #                     message="Loading ob data ...",
        #                     status=StatusMessageStatus.working)
        # o_norm.load(data=ob, data_type=DataType.ob)
        # show_status_message(parent=self.parent,
        #                     message="Loading ob data ... Done!",
        #                     status=StatusMessageStatus.working,
        #                     duration_s=5)

        list_roi_object = []
        for _roi in list_roi:
            o_roi = ROI(
                x0=int(_roi[0]),
                y0=int(_roi[1]),
                width=int(_roi[2]),
                height=int(_roi[3]),
            )
            list_roi_object.append(o_roi)

        show_status_message(
            parent=self.parent,
            message="Running normalization ...",
            status=StatusMessageStatus.working,
        )
        if list_roi_object:
            o_norm.normalization(roi=list_roi_object)
        else:
            o_norm.normalization()

        # self.o_norm = o_norm
        #
        # show_status_message(parent=self.parent,
        #                     message="Running normalization ... Done!",
        #                     status=StatusMessageStatus.working,
        #                     duration_s=5)
        #
        # show_status_message(parent=self.parent,
        #                     message="Exporting normalized files ...",
        #                     status=StatusMessageStatus.working)
        # o_norm.export(folder=output_folder)
        # show_status_message(parent=self.parent,
        #                     message="Exporting normalized files ... Done!",
        #                     status=StatusMessageStatus.working,
        #                     duration_s=5)

        logger.info(" running normalization with sample and ob data ... Done!")
        return o_norm

    # def can_we_use_buffered_data(self, kernel_dimension=None, kernel_size=None, kernel_type=None):
    #     if self.parent.moving_average_config is None:
    #         return False
    #
    #     buffered_kernel_dimension = self.parent.moving_average_config.get('kernel_dimension', None)
    #     if buffered_kernel_dimension is None:
    #         return False
    #     if not (kernel_dimension == buffered_kernel_dimension):
    #         return False
    #
    #     buffered_kernel_size = self.parent.moving_average_config.get('kernel_size', None)
    #     if buffered_kernel_size is None:
    #         return False
    #     if not (buffered_kernel_size['x'] == kernel_size['x']):
    #         return False
    #     if not (buffered_kernel_size['y'] == kernel_size['y']):
    #         return False
    #     if kernel_dimension == '3d':
    #         if not (buffered_kernel_size['lambda'] == kernel_size['lambda']):
    #             return False
    #
    #     buffered_kernel_type = self.parent.moving_average_config.get('kernel_type', None)
    #     if buffered_kernel_type is None:
    #         return False
    #     if not (buffered_kernel_type == kernel_type):
    #         return False
    #
    #     return True



from ibeatles.session import SessionSubKeys

TO_MICROS_UNITS = 1e6
TO_ANGSTROMS_UNITS = 1e10


class BinAutoMode:
    log = "log"
    linear = "linear"


class BinMode:
    """list of mode to bin the data"""

    auto = "auto"
    manual = "manual"
    settings = "settings"


class BinAlgorithm:
    """list of algorithm used to bin the images"""

    mean = "mean"
    median = "median"


class StatisticsName:
    mean = "mean"
    median = "median"
    std = "std"
    min = "min"
    max = "max"


class StatisticsRegion:
    full = "full"
    roi = "roi"


session = {
    SessionSubKeys.top_folder: None,  # the base folder to start looking at images folder to combine
    SessionSubKeys.list_working_folders: None,  # list of working folders
    SessionSubKeys.list_working_folders_status: None,  # list of working folders status [True, True, False..]
    SessionSubKeys.log_buffer_size: 500,  # max size of the log file
    SessionSubKeys.version: "0.0.1",  # version of that config
    SessionSubKeys.distance_source_detector: 25.0,
    SessionSubKeys.detector_offset: 9600,
    SessionSubKeys.sample_position: 0,  # in the combine tab
    SessionSubKeys.bin_mode: BinMode.auto,  # 'auto' or 'manual',
}



#!/usr/bin/env python
"""
Event handler module
"""

import copy
import logging
import os

import numpy as np
from qtpy.QtWidgets import QCheckBox, QFileDialog

from ibeatles import DataType, interact_me_style, normal_style

# MVP widget
from ibeatles.app.presenters.time_spectra_presenter import TimeSpectraPresenter

# backend function from core
from ibeatles.core.io.data_loading import get_time_spectra_filename
from ibeatles.session import SessionSubKeys
from ibeatles.tools.tof_combine import ANGSTROMS, LAMBDA, MICRO
from ibeatles.tools.tof_combine import SessionKeys as TofCombineSessionKeys
from ibeatles.tools.tof_combine.combine.combine import Combine
from ibeatles.tools.tof_combine.load.load_files import LoadFiles
from ibeatles.tools.tof_combine.utilities.get import Get as TofCombineGet
from ibeatles.tools.tof_combine.utilities.table_handler import TableHandler
from ibeatles.tools.utilities import TimeSpectraKeys
from ibeatles.utilities.file_handler import FileHandler
from ibeatles.utilities.status_message_config import (
    StatusMessageStatus,
    show_status_message,
)


class EventHandler:
    no_data_loaded = False

    def __init__(self, parent=None, grand_parent=None):
        self.parent = parent
        self.grand_parent = grand_parent

        self.time_spectra_presenter = None

    def visualize_flag_changed(self):
        self.parent.visualize_flag = self.parent.ui.visualize_checkBox.isChecked()
        if self.parent.visualize_flag:
            self.parent.ui.combine_horizontal_splitter.setSizes([20, 500])
        else:
            self.parent.ui.combine_horizontal_splitter.setSizes([100, 0])

    def select_top_folder(self):
        if self.no_data_loaded:
            return

        default_path = self.grand_parent.session_dict[DataType.sample][SessionSubKeys.current_folder]
        folder = str(
            QFileDialog.getExistingDirectory(
                caption="Select Top Working Folder",
                directory=default_path,
                options=QFileDialog.ShowDirsOnly,
            )
        )
        if folder == "":
            logging.info("User Canceled the selection of top folder dialog!")
            return

        logging.info(f"Users selected a new top folder: {folder}")

        # get list of folders in top folder
        list_folders = FileHandler.get_list_of_all_subfolders(folder)
        list_folders = self.keep_only_folders_with_tiff_files(list_folders)
        list_folders.sort()

        self.parent.session[TofCombineSessionKeys.list_folders] = list_folders
        self.parent.session[TofCombineSessionKeys.top_folder] = folder

        # initialize parameters when using new working folder
        self.reset_data()

        # display the full path of the top folder selected
        self.parent.ui.top_folder_label.setText(folder)

        # display list of folders in widget and column showing working folders used
        self.populate_list_of_folders_to_combine()

        # update ui
        self.check_widgets()

    def at_least_one_folder_selected(self):
        o_table = TableHandler(table_ui=self.parent.ui.combine_tableWidget)
        nbr_row = o_table.row_count()
        for _row in np.arange(nbr_row):
            _horizontal_widget = o_table.get_widget(row=_row, column=0)
            radio_button = _horizontal_widget.layout().itemAt(1).widget()
            if radio_button.isChecked():
                return True
        return False

    def at_least_two_folder_selected(self):
        """check if there are at least 2 folders selected"""
        nbr_folder_selected = 0
        o_table = TableHandler(table_ui=self.parent.ui.combine_tableWidget)
        nbr_row = o_table.row_count()
        for _row in np.arange(nbr_row):
            _horizontal_widget = o_table.get_widget(row=_row, column=0)
            radio_button = _horizontal_widget.layout().itemAt(1).widget()
            if radio_button.isChecked():
                nbr_folder_selected += 1

        if nbr_folder_selected >= 2:
            return True

        return False

    def check_widgets(self):
        o_table = TableHandler(table_ui=self.parent.ui.combine_tableWidget)
        nbr_row = o_table.row_count()
        if nbr_row == 0:
            self.parent.ui.combine_select_top_folder_pushButton.setStyleSheet(interact_me_style)
        else:
            self.parent.ui.combine_select_top_folder_pushButton.setStyleSheet(normal_style)

        if self.parent.session[TofCombineSessionKeys.top_folder]:
            self.parent.ui.combine_refresh_top_folder_pushButton.setEnabled(True)
            self.parent.ui.combine_refresh_top_folder_pushButton.setStyleSheet(interact_me_style)
        else:
            self.parent.ui.combine_refresh_top_folder_pushButton.setEnabled(False)
            self.parent.ui.combine_refresh_top_folder_pushButton.setStyleSheet(normal_style)

        if self.at_least_two_folder_selected():
            self.parent.ui.combine_pushButton.setEnabled(True)
            self.parent.ui.combine_pushButton.setStyleSheet(interact_me_style)
        else:
            self.parent.ui.combine_widget.setEnabled(False)
            self.parent.ui.combine_pushButton.setEnabled(False)
            self.parent.ui.combine_pushButton.setStyleSheet(normal_style)

        if self.at_least_one_folder_selected():
            # enable display widgets
            self.parent.ui.combine_widget.setEnabled(True)
        else:
            self.parent.ui.combine_widget.setEnabled(False)

    def refresh_table_clicked(self):
        self.parent.ui.combine_tableWidget.blockSignals(True)

        logging.info("User clicked the refresh table!")
        top_folder = self.parent.session[TofCombineSessionKeys.top_folder]
        list_folders = FileHandler.get_list_of_all_subfolders(top_folder)
        list_folders = self.keep_only_folders_with_tiff_files(list_folders=list_folders)

        # checking if there is any new folder
        current_list_of_folders = []
        for _row in self.parent.dict_data_folders.keys():
            current_list_of_folders.append(self.parent.dict_data_folders[_row][TofCombineSessionKeys.folder])

        row = len(current_list_of_folders)
        for _folder in list_folders:
            if _folder not in current_list_of_folders:
                list_folders.append(_folder)
                list_files = FileHandler.get_list_of_tif(_folder)
                nbr_files = len(list_files)

                self.parent.dict_data_folders[row] = {
                    TofCombineSessionKeys.folder: _folder,
                    TofCombineSessionKeys.data: None,
                    TofCombineSessionKeys.list_files: list_files,
                    TofCombineSessionKeys.nbr_files: nbr_files,
                    TofCombineSessionKeys.use: False,
                }
                self.insert_row_entry(row)
                row += 1

        self.parent.session[TofCombineSessionKeys.list_folders] = list_folders
        self.parent.ui.combine_tableWidget.blockSignals(False)

    def keep_only_folders_with_tiff_files(self, list_folders):
        """will go over the list of folders and will only keep the one with tiff in it"""
        list_folders.sort()
        clean_list_folders = []
        for _folder in list_folders:
            list_files = FileHandler.get_list_of_tif(_folder)
            if len(list_files) > 0:
                clean_list_folders.append(_folder)

        return clean_list_folders

    def reset_data(self):
        """
        This re-initialize all the parameters when working with a new top folder
        """

        # reset master dictionary that contains the raw data
        list_folders = self.parent.session[TofCombineSessionKeys.list_folders]
        if list_folders is None:
            return

        _data_dict = {}
        for _row, _folder in enumerate(list_folders):
            list_files = FileHandler.get_list_of_tif(_folder)
            nbr_files = len(list_files)
            _data_dict[_row] = {
                TofCombineSessionKeys.folder: _folder,
                TofCombineSessionKeys.data: None,
                TofCombineSessionKeys.list_files: list_files,
                TofCombineSessionKeys.nbr_files: nbr_files,
                TofCombineSessionKeys.use: False,
            }
        self.parent.dict_data_folders = _data_dict

        # reset time spectra
        self.parent.time_spectra = {
            TimeSpectraKeys.file_name: None,
            TimeSpectraKeys.tof_array: None,
            TimeSpectraKeys.lambda_array: None,
            TimeSpectraKeys.file_index_array: None,
        }

        self._reset_time_spectra_tab()

    def _reset_time_spectra_tab(self):
        self.parent.ui.time_spectra_name_label.setText("N/A")
        self.parent.ui.time_spectra_preview_pushButton.setEnabled(False)

    def populate_list_of_folders_to_combine(self):
        list_of_folders = self.parent.session[TofCombineSessionKeys.list_folders]

        o_table = TableHandler(table_ui=self.parent.ui.combine_tableWidget)
        o_table.remove_all_rows()

        if list_of_folders is None:
            return

        for _row, _folder in enumerate(list_of_folders):
            self.insert_row_entry(_row)

    def insert_row_entry(self, row=0):
        folder_dict = self.parent.dict_data_folders[row]

        status_of_folder = folder_dict[TofCombineSessionKeys.use]
        nbr_files = folder_dict[TofCombineSessionKeys.nbr_files]
        folder_name = folder_dict[TofCombineSessionKeys.folder]

        o_table = TableHandler(table_ui=self.parent.ui.combine_tableWidget)
        row = o_table.row_count()
        o_table.insert_empty_row(row=row)

        # use or not that row
        check_box = QCheckBox()
        check_box.setChecked(status_of_folder)

        # check if this file has more than 1 file
        # if not disable radio button
        # number of images in that folder
        if nbr_files > 0:
            check_box.setEnabled(True)
        else:
            check_box.setEnabled(False)

        o_table.insert_widget(row=row, column=0, widget=check_box, centered=True)
        check_box.clicked.connect(self.parent.radio_buttons_of_folder_changed)

        o_table.insert_item(row=row, column=1, value=nbr_files, editable=False)

        # full path of the folder
        o_table.insert_item(row=row, column=2, value=folder_name, editable=False)

    def update_list_of_folders_to_use(self, force_recalculation_of_time_spectra=False):
        o_table = TableHandler(table_ui=self.parent.ui.combine_tableWidget)
        nbr_row = o_table.row_count()
        for _row_index in np.arange(nbr_row):
            _horizontal_widget = o_table.get_widget(row=_row_index, column=0)
            radio_button = _horizontal_widget.layout().itemAt(1).widget()
            self.parent.dict_data_folders[_row_index][TofCombineSessionKeys.use] = radio_button.isChecked()

        for _row_index in np.arange(nbr_row):
            if self.parent.dict_data_folders[_row_index][TofCombineSessionKeys.use]:
                _folder_name = self.parent.dict_data_folders[_row_index][TofCombineSessionKeys.folder]

                if force_recalculation_of_time_spectra:
                    self.load_time_spectra_file(folder=_folder_name)
                    self.fix_linear_bin_radio_button_max_values()

                if self.parent.dict_data_folders[_row_index][TofCombineSessionKeys.data] is None:
                    _ = self.load_that_folder(folder_name=_folder_name)

                    # load time spectra if not already there
                    if self.parent.time_spectra[TimeSpectraKeys.file_name] is None:
                        self.load_time_spectra_file(folder=_folder_name)
                        self.fix_linear_bin_radio_button_max_values()

    def load_time_spectra_file(self, folder=None):
        """
        load the time spectra file

        Parameters
        ----------
        folder: str
            location of the time spectra file

        Returns
        -------
        None
        """
        time_spectra_file = get_time_spectra_filename(folder)
        if not time_spectra_file:
            logging.info("Time spectra file not found!")
            show_status_message(
                parent=self.parent,
                message="Time spectra file not found!",
                status=StatusMessageStatus.error,
                duration_s=5,
            )
            return

        if self.time_spectra_presenter is None:
            self.time_spectra_presenter = TimeSpectraPresenter(self.parent)

        distance_source_detector_m = float(self.grand_parent.ui.distance_source_detector.text())
        detector_offset = float(self.grand_parent.ui.detector_offset.text())

        try:
            self.time_spectra_presenter.load_data(time_spectra_file, distance_source_detector_m, detector_offset)
            self.update_time_spectra_data()
        except Exception as e:
            logging.error(f"Error loading time spectra: {str(e)}")
            show_status_message(
                parent=self.parent,
                message=f"Error loading time spectra: {str(e)}",
                status=StatusMessageStatus.error,
                duration_s=5,
            )

    def update_time_spectra_data(self):
        time_spectra_data = self.time_spectra_presenter.model.get_data()

        self.parent.time_spectra[TimeSpectraKeys.file_name] = time_spectra_data["filename"]
        self.parent.time_spectra[TimeSpectraKeys.tof_array] = time_spectra_data["tof_array"]
        self.parent.time_spectra[TimeSpectraKeys.lambda_array] = time_spectra_data["lambda_array"]
        self.parent.time_spectra[TimeSpectraKeys.file_index_array] = np.arange(len(time_spectra_data["tof_array"]))
        self.parent.time_spectra[TimeSpectraKeys.counts_array] = time_spectra_data["counts_array"]

        # update time spectra tab
        self.parent.ui.time_spectra_name_label.setText(os.path.basename(time_spectra_data["filename"]))
        self.parent.ui.time_spectra_preview_pushButton.setEnabled(True)

    def load_that_folder(self, folder_name=None):
        """
        this routine load all the images of the selected folder
        :param folder_name: full path of the folder containing the images to load
        :return: True if the loading worked, False otherwise
        """
        if not os.path.exists(folder_name):
            logging.info(f"Unable to load data from folder {folder_name}")
            return False

        # load the data
        o_load = LoadFiles(parent=self.parent, folder=folder_name)
        data = o_load.retrieve_data()

        o_get = TofCombineGet(parent=self.parent)
        row = o_get.row_of_that_folder(folder=folder_name)
        self.parent.dict_data_folders[row][TofCombineSessionKeys.data] = data

        return True

    def combine_algorithm_changed(self):
        if self.no_data_loaded:
            return

        o_get = TofCombineGet(parent=self.parent)
        combine_algorithm = o_get.combine_algorithm()

        self.parent.session[TofCombineSessionKeys.combine_algorithm] = combine_algorithm
        logging.info(f"Algorithm to combine changed to: {combine_algorithm}")
        self.combine_folders()
        self.display_profile()

    def combine_folders(self):
        if self.no_data_loaded:
            return

        o_combine = Combine(parent=self.parent)
        o_combine.run()

    def combine_roi_changed(self):
        live_combine_image = self.parent.live_combine_image
        image_view = self.parent.combine_image_view
        roi_item = self.parent.combine_roi_item_id

        region = roi_item.getArraySlice(live_combine_image, image_view.imageItem)
        x0 = region[0][0].start
        x1 = region[0][0].stop - 1
        y0 = region[0][1].start
        y1 = region[0][1].stop - 1

        width = x1 - x0
        height = y1 - y0

        self.parent.combine_roi = {"x0": x0, "y0": y0, "width": width, "height": height}

    def display_profile(self):
        if self.no_data_loaded:
            return

        combine_data = self.parent.combine_data

        if combine_data is None:
            self.parent.combine_profile_view.clear()
            return

        x0 = self.parent.combine_roi["x0"]
        y0 = self.parent.combine_roi["y0"]
        width = self.parent.combine_roi["width"]
        height = self.parent.combine_roi["height"]

        o_get = TofCombineGet(parent=self.parent)
        combine_algorithm = o_get.combine_algorithm()
        time_spectra_x_axis_name = o_get.combine_x_axis_selected()

        profile_signal = [np.mean(_data[y0 : y0 + height, x0 : x0 + width]) for _data in combine_data]
        # if combine_algorithm == CombineAlgorithm.mean:
        #     profile_signal = [np.mean(_data[y0:y0+height, x0:x0+width]) for _data in combine_data]
        # elif combine_algorithm == CombineAlgorithm.median:
        #     profile_signal = [np.median(_data[y0:y0+height, x0:x0+width]) for _data in combine_data]
        # else:
        #     raise NotImplementedError("Combine algorithm not implemented!")

        self.parent.profile_signal = profile_signal
        self.parent.combine_profile_view.clear()
        x_axis = copy.deepcopy(self.parent.time_spectra[time_spectra_x_axis_name])

        if time_spectra_x_axis_name == TimeSpectraKeys.file_index_array:
            x_axis_label = "file index"
        elif time_spectra_x_axis_name == TimeSpectraKeys.tof_array:
            x_axis *= 1e6  # to display axis in micros
            x_axis_label = "tof (" + MICRO + "s)"
        elif time_spectra_x_axis_name == TimeSpectraKeys.lambda_array:
            x_axis *= 1e10  # to display axis in Angstroms
            x_axis_label = LAMBDA + "(" + ANGSTROMS + ")"

        self.parent.combine_profile_view.plot(x_axis, profile_signal, pen="r", symbol="x")
        self.parent.combine_profile_view.setLabel("left", f"{combine_algorithm} counts")
        self.parent.combine_profile_view.setLabel("bottom", x_axis_label)

    def fix_linear_bin_radio_button_max_values(self):
        time_spectra = self.parent.time_spectra

        file_index_array = time_spectra[TimeSpectraKeys.file_index_array]
        max_file_index = int(len(file_index_array))

        tof_array = time_spectra[TimeSpectraKeys.tof_array]
        max_tof = float(tof_array[-1]) * 1e6
        min_tof = float(tof_array[0]) * 1e6

        lambda_array = time_spectra[TimeSpectraKeys.lambda_array]
        max_lambda = float(lambda_array[-1]) * 1e10
        min_lambda = float(lambda_array[0]) * 1e10

        logging.info("Max values of bin linear spin box has been fixed:")
        logging.info(f"-> file index: {max_file_index}")
        logging.info(f"-> tof: {min_tof =} to {max_tof =}")
        logging.info(f"-> lambda: {min_lambda =} to {max_lambda =}")



#!/usr/bin/env python
"""
Event handler for the step3
"""

from loguru import logger

from ibeatles import DataType
from ibeatles.all_steps.event_handler import EventHandler as TopEventHandler
from ibeatles.step1.data_handler import DataHandler
from ibeatles.step1.plot import Step1Plot
from ibeatles.step3.gui_handler import Step3GuiHandler
from ibeatles.utilities.retrieve_data_infos import RetrieveGeneralDataInfos


class EventHandler(TopEventHandler):
    def import_button_clicked(self):
        logger.info(f"{self.data_type} import button clicked")

        self.parent.loading_flag = True
        o_load = DataHandler(parent=self.parent, data_type=self.data_type)
        _folder = o_load.select_folder()
        state = o_load.import_files_from_folder(folder=_folder, extension=[".tif", ".fits", ".tiff"])

        if state:
            o_load.import_time_spectra()

            if self.parent.data_metadata[self.data_type]["data"] is not None:
                self.update_ui_after_loading_data(folder=_folder)

            self.check_time_spectra_status()
            self.parent.infos_window_update(data_type=self.data_type)

            self.parent.ui.normalized_splitter.setSizes([20, 450])

    def sample_list_selection_changed(self):
        if not self.parent.loading_flag:
            o_retrieve_data_infos = RetrieveGeneralDataInfos(parent=self.parent, data_type=DataType.normalized)
            o_retrieve_data_infos.update()
            self.parent.roi_normalized_image_view_changed(mouse_selection=False)
        else:
            self.parent.loading_flag = False

    def import_button_clicked_automatically(self, folder=None):
        o_load = DataHandler(parent=self.parent, data_type=self.data_type)
        o_load.import_files_from_folder(folder=folder, extension=[".tif", ".fits", ".tiff"])
        o_load.import_time_spectra()

        if self.parent.data_metadata[self.data_type]["data"].any():
            self.update_ui_after_loading_data(folder=folder)

    def update_ui_after_loading_data(self, folder=None):
        self.parent.data_metadata[self.data_type]["folder"] = folder
        self.parent.select_load_data_row(data_type=self.data_type, row=0)
        self.parent.retrieve_general_infos(data_type=self.data_type)
        self.parent.retrieve_general_data_infos(data_type=self.data_type)
        o_plot = Step1Plot(parent=self.parent, data_type=self.data_type)
        o_plot.initialize_default_roi()
        o_plot.display_bragg_edge(mouse_selection=False)
        o_gui = Step3GuiHandler(parent=self.parent)
        o_gui.check_widgets()
        # self.check_time_spectra_status()

    def check_time_spectra_status(self):
        if str(self.parent.ui.time_spectra.text()):
            self.parent.ui.display_warning.setVisible(False)
        else:
            self.parent.ui.display_warning.setVisible(True)



#!/usr/bin/env python
"""
FittingParametersViewerEditorHandler class for handling the fitting parameters viewer and editor.
"""

import numpy as np
from qtpy import QtCore, QtGui
from qtpy.QtGui import QBrush
from qtpy.QtWidgets import QTableWidgetItem

from ibeatles.fitting.kropff import SessionSubKeys
from ibeatles.fitting.kropff.get import Get
from ibeatles.utilities.bins import convert_bins_to_keys
from ibeatles.utilities.table_handler import TableHandler


class FittingParametersViewerEditorHandler:
    colorscale_nbr_row = 15
    colorscale_cell_size = {"width": 75, "height": 30}

    def __init__(self, grand_parent=None, parent=None):
        self.grand_parent = grand_parent
        self.parent = parent

        self.table_dictionary = self.grand_parent.kropff_table_dictionary

    def get_variable_selected(self):
        if self.parent.ui.lambda_hkl_button.isChecked():
            return SessionSubKeys.lambda_hkl
        elif self.parent.ui.sigma_button.isChecked():
            return SessionSubKeys.sigma
        elif self.parent.ui.tau_button.isChecked():
            return SessionSubKeys.tau

    def populate_table_with_variable(self, variable=None):
        o_get = Get(parent=self.parent)
        if variable is None:
            variable = o_get.variable_selected()

        array_2d_values = self.create_array_of_variable(variable=variable)

        # retrieve min and max values
        # if np.isnan(array_2d_values).any():
        #     print("#1")
        #     min_value = np.nan
        #     max_value = np.nan
        #     mid_point = np.nan
        # else:
        # print("#2")
        min_value = np.nanmin(array_2d_values)
        max_value = np.nanmax(array_2d_values)
        mid_point = np.mean([min_value, max_value])

        # define colorscale table
        self.initialize_colorscale_table(min_value=min_value, max_value=max_value)

        [nbr_row, nbr_column] = np.shape(array_2d_values)
        for _row in np.arange(nbr_row):
            for _col in np.arange(nbr_column):
                _value = array_2d_values[_row, _col]
                _color = self.get_color_for_this_value(min_value=min_value, max_value=max_value, value=_value)
                if np.isnan(_value):
                    _item = QTableWidgetItem("nan")
                else:
                    if variable == SessionSubKeys.sigma:
                        _item = QTableWidgetItem("{:04.6f}".format(_value))
                    else:
                        _item = QTableWidgetItem("{:04.4f}".format(_value))
                _item.setBackground(_color)

                bin_index = _row + nbr_row * _col
                if self.is_bin_locked(bin_index=bin_index):
                    _gradient = QtGui.QRadialGradient(10, 10, 10, 20, 20)
                    _gradient.setColorAt(1, _color)
                    # if self.is_bin_fixed(bin_index=bin_index, variable_name=variable):
                    #     _gradient.setColorAt(0.5, QtGui.QColor(255, 255, 255))
                    _gradient.setColorAt(0, QtGui.QColor(255, 0, 0, alpha=255))
                    _item.setBackground(QtGui.QBrush(_gradient))
                    _item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable)

                # elif self.is_bin_activated(bin_index=bin_index):
                #     _gradient = QtGui.QRadialGradient(10, 10, 10, 20, 20)
                #     _gradient.setColorAt(1, _color)
                #     # if self.is_bin_fixed(bin_index=bin_index, variable_name=variable):
                #     #     _gradient.setColorAt(0.5, QtGui.QColor(255, 255, 255))
                #     _gradient.setColorAt(0, QtGui.QColor(0, 0, 255, alpha=255))
                #     _item.setBackground(QtGui.QBrush(_gradient))
                # # else:
                # #     if self.is_bin_fixed(bin_index=bin_index, variable_name=variable):
                # #         _gradient = QtGui.QRadialGradient(10, 10, 10, 20, 20)
                # #         _gradient.setColorAt(1, _color)
                # #         _gradient.setColorAt(0, QtGui.QColor(255, 255, 255))
                # #         _item.setBackground(QtGui.QBrush(_gradient))
                #
                if _value > mid_point:
                    _foreground_color = QtGui.QColor(255, 255, 255, alpha=255)
                    _item.setForeground(QBrush(_foreground_color))

                self.parent.ui.variable_table.blockSignals(True)
                self.parent.ui.variable_table.setItem(_row, _col, _item)
                self.parent.ui.variable_table.blockSignals(False)

    def is_bin_fixed(self, bin_index=0, variable_name="lambda_hkl"):
        return self.table_dictionary[str(bin_index)][variable_name]["fixed"]

    def is_bin_locked(self, bin_index=0):
        return self.table_dictionary[str(bin_index)]["lock"]

    def is_bin_activated(self, bin_index=0):
        return self.table_dictionary[str(bin_index)]["active"]

    def clear_colorscale_table(self):
        nbr_row = self.parent.colorscale_table.rowCount()
        for _row in np.arange(nbr_row):
            self.parent.colorscale_table.removeRow(0)

    def initialize_colorscale_table(self, min_value=0, max_value=1):
        self.clear_colorscale_table()

        nbr_row = self.colorscale_nbr_row
        step = (float(max_value) - float(min_value)) / (nbr_row - 1)
        mid_point = int(nbr_row / 2)
        _row = 0

        if min_value == max_value:
            nbr_row = 1

        if np.isnan(step):
            nbr_row = 1

        for _index in np.arange(nbr_row - 1, -1, -1):
            self.parent.colorscale_table.insertRow(_row)
            self.parent.colorscale_table.setRowHeight(_row, self.colorscale_cell_size["height"])
            self.parent.colorscale_table.setColumnWidth(_row, self.colorscale_cell_size["width"])
            if np.isnan(step):
                _value = np.nan
            else:
                _value = min_value + _index * step
            _color = self.get_color_for_this_value(min_value=min_value, max_value=max_value, value=_value)

            if np.isnan(_value):
                _item = QTableWidgetItem("nan")
            else:
                o_get = Get(parent=self.parent)
                if o_get.variable_selected() == SessionSubKeys.sigma:
                    _item = QTableWidgetItem("{:04.6f}".format(_value))
                else:
                    _item = QTableWidgetItem("{:04.4f}".format(_value))

            _item.setBackground(_color)
            _item.setTextAlignment(QtCore.Qt.AlignRight)
            if (_row < mid_point) and (nbr_row != 1):
                # font should be changed from black to white to be visible over the very dark background
                _foreground_color = QtGui.QColor(255, 255, 255, alpha=255)
                _item.setForeground(QBrush(_foreground_color))

            self.parent.colorscale_table.setItem(_row, 0, _item)
            _row += 1

    def get_color_for_this_value(self, min_value=0, max_value=1, value=0):
        if np.isnan(value):
            return QtGui.QColor(255, 255, 255, alpha=100)  # white
        elif max_value == min_value:
            return QtGui.QColor(250, 0, 0, alpha=255)  # red

        _ratio = 1 - (float(value) - float(min_value)) / (float(max_value) - float(min_value))
        return QtGui.QColor(0, int(_ratio * 255), 0, alpha=255)

    def create_array_of_variable(self, variable=None):
        if variable is None:
            o_get = Get(parent=self.parent)
            variable = o_get.variable_selected()

        table_dictionary = self.grand_parent.kropff_table_dictionary
        _table_selection = self.grand_parent.fitting_selection
        nbr_column = _table_selection["nbr_column"]
        nbr_row = _table_selection["nbr_row"]

        _array = np.zeros((nbr_row, nbr_column), dtype=float)

        for _entry in table_dictionary.keys():
            row_index = table_dictionary[_entry]["row_index"]
            column_index = table_dictionary[_entry]["column_index"]
            value = table_dictionary[_entry][variable]["val"]
            _array[row_index, column_index] = value

        return _array

    def set_new_value_to_selected_bins(
        self, selection=[], variable_name="d_spacing", variable_value=0, table_nbr_row=0
    ):
        table_dictionary = self.table_dictionary
        nbr_row = table_nbr_row

        for _select in selection:
            _left_column = _select.leftColumn()
            _right_column = _select.rightColumn()
            _top_row = _select.topRow()
            _bottom_row = _select.bottomRow()
            for _row in np.arange(_top_row, _bottom_row + 1):
                for _col in np.arange(_left_column, _right_column + 1):
                    _index = str(_row + _col * nbr_row)
                    if not table_dictionary[_index]["lock"]:
                        table_dictionary[_index][variable_name]["val"] = float(variable_value)
                        table_dictionary[_index][variable_name]["err"] = np.nan
            self.grand_parent.fitting_set_variables_ui.ui.variable_table.setRangeSelected(_select, False)

        self.grand_parent.kropff_table_dictionary = table_dictionary
        self.populate_table_with_variable(variable=variable_name)

    def variable_cell_manual_changed(self, row=-1, column=-1):
        o_get = Get(parent=self.parent)
        variable_selected = o_get.variable_selected()

        key = convert_bins_to_keys(list_of_bins=[(row, column)], full_bin_height=self.parent.nbr_row)

        o_table = TableHandler(table_ui=self.parent.ui.variable_table)
        cell_value = o_table.get_item_float_from_cell(row=row, column=column)

        table_dictionary = self.parent.kropff_table_dictionary
        table_dictionary[key[0]][variable_selected]["val"] = cell_value

        self.parent.kropff_table_dictionary = table_dictionary
        self.grand_parent.kropff_table_dictionary = table_dictionary
        self.populate_table_with_variable(variable=variable_selected)
        o_table.select_everything(False)



#!/usr/bin/env python
"""
Initialization (step 1)
"""

import pyqtgraph as pg
from neutronbraggedge.material_handler.retrieve_material_metadata import (
    RetrieveMaterialMetadata,
)
from pyqtgraph.dockarea import Dock, DockArea
from qtpy import QtCore
from qtpy.QtCore import QSize
from qtpy.QtGui import QIcon, QPixmap
from qtpy.QtWidgets import (
    QHBoxLayout,
    QLabel,
    QProgressBar,
    QPushButton,
    QRadioButton,
    QScrollBar,
    QSizePolicy,
    QSpacerItem,
    QVBoxLayout,
    QWidget,
)

from ibeatles import (
    DataType,
    error_icon_file,
    fitting_image,
    infos_file,
    pixel_binning_image,
    preview_file,
    rotate_image,
    step1_icon,
    step2_icon,
    step3_icon,
    step4_icon,
    strain_mapping_image,
    tof_binning_image,
    tof_combine_image,
)
from ibeatles.step1.roi import Roi
from ibeatles.utilities.table_handler import TableHandler

tab6_top_button_width = 250
tab6_top_button_height = 150

tab6_bottom_buttom_width = 250
tab6_bottom_button_height = 250


class Initialization:
    def __init__(self, parent=None):
        self.parent = parent

    def all(self):
        self.gui()
        self.labels()
        self.material_widgets()
        self.statusbar()
        self.pyqtgraph()
        self.icons()
        self.widgets()
        self.splitters()
        self.tabs()

    def tabs(self):
        self.parent.ui.tabWidget.setTabIcon(0, QIcon(step1_icon))
        self.parent.ui.tabWidget.setTabIcon(1, QIcon(step2_icon))
        self.parent.ui.tabWidget.setTabIcon(2, QIcon(step3_icon))
        self.parent.ui.tabWidget.setTabIcon(3, QIcon(step4_icon))

    def splitters(self):
        self.parent.ui.sample_ob_splitter.setSizes([100, 0])
        self.parent.ui.sample_ob_splitter.setStyleSheet("""
                                     QSplitter::handle{
                                     image: url(":/MPL Toolbar/horizontal_splitter_handle.png");
                                     }
                                     """)
        self.parent.ui.sample_ob_splitter.setHandleWidth(15)

        self.parent.ui.horizontal_normalization_splitter.setStyleSheet("""
                                     QSplitter::handle{
                                     image: url(":/MPL Toolbar/horizontal_splitter_handle.png");
                                     }
                                     """)

        self.parent.ui.normalized_splitter.setSizes([100, 0])
        self.parent.ui.normalized_splitter.setStyleSheet("""
                                     QSplitter::handle{
                                     image: url(":/MPL Toolbar/horizontal_splitter_handle.png");
                                     }
                                     """)
        self.parent.ui.normalized_splitter.setHandleWidth(15)

        self.parent.ui.area.setStyleSheet("""
                                     QSplitter::handle{
                                     image: url(":/MPL Toolbar/vertical_splitter_handle.png");
                                     }
                                     """)

        self.parent.ui.ob_area.setStyleSheet("""
                                         QSplitter::handle{
                                         image: url(":/MPL Toolbar/vertical_splitter_handle.png");
                                         }
                                         """)
        self.parent.ui.normalized_area.setStyleSheet("""
                                         QSplitter::handle{
                                         image: url(":/MPL Toolbar/vertical_splitter_handle.png");
                                         }
                                         """)

    def widgets(self):
        # folder path of time spectra
        self.parent.ui.time_spectra_folder_2.setVisible(False)
        self.parent.ui.time_spectra_folder.setVisible(False)

        self.parent.ui.sample_infos_pushButton.setIcon(QIcon(infos_file))
        self.parent.ui.ob_infos_pushButton.setIcon(QIcon(infos_file))
        self.parent.ui.normalized_infos_pushButton.setIcon(QIcon(infos_file))

        column_sizes = [30, 30, 30, 60]
        o_table = TableHandler(table_ui=self.parent.ui.pre_defined_tableWidget)
        o_table.set_column_sizes(column_sizes=column_sizes)
        o_table = TableHandler(table_ui=self.parent.ui.method2_tableWidget)
        o_table.set_column_sizes(column_sizes=column_sizes)
        o_table = TableHandler(table_ui=self.parent.ui.method1_tableWidget)
        o_table.set_column_sizes(column_sizes=column_sizes)

    def statusbar(self):
        self.parent.eventProgress = QProgressBar(self.parent.ui.statusbar)
        self.parent.eventProgress.setMinimumSize(20, 14)
        self.parent.eventProgress.setMaximumSize(540, 100)
        self.parent.eventProgress.setVisible(False)
        self.parent.ui.statusbar.addPermanentWidget(self.parent.eventProgress)
        self.parent.setStyleSheet("QStatusBar{padding-left:8px;color:red;font-weight:bold;}")

    def gui(self):
        # define position and size
        rect = self.parent.geometry()
        self.parent.setGeometry(10, 10, rect.width(), rect.height())
        self.parent.ui.sample_ob_splitter.setSizes([850, 20])
        self.parent.ui.normalized_splitter.setSizes([150, 600])

        # ob tab
        self.parent.ui.load_data_tab.blockSignals(True)
        self.parent.ui.load_data_tab.setTabEnabled(1, False)
        self.parent.ui.load_data_tab.blockSignals(False)

        # normalized
        self.parent.ui.tabWidget.setTabEnabled(1, False)

        # add shortcuts to menu button
        self.parent.ui.action1_load_data.setShortcut("Ctrl+1")
        self.parent.ui.action2_Normalization_2.setShortcut("Ctrl+2")
        self.parent.ui.action3_Normalized_Data.setShortcut("Ctrl+3")
        self.parent.ui.action3_Binning.setShortcut("Ctrl+4")
        self.parent.ui.action4_Fitting.setShortcut("Ctrl+5")
        self.parent.ui.action5_Results.setShortcut("Ctrl+6")

    def material_widgets(self):
        retrieve_material = RetrieveMaterialMetadata(material="all")
        list_returned = retrieve_material.full_list_material()

        self.parent.ui.pre_defined_list_of_elements.blockSignals(True)
        self.parent.ui.user_defined_list_of_elements.blockSignals(True)
        self.parent.ui.pre_defined_list_of_elements.addItems(list_returned)
        extanded_list = list(list_returned)
        extanded_list.insert(0, "None")
        self.parent.ui.user_defined_list_of_elements.addItems(extanded_list)
        self.parent.ui.pre_defined_list_of_elements.blockSignals(False)
        self.parent.ui.user_defined_list_of_elements.blockSignals(False)

        # o_gui = GuiHandler(parent=self.parent,
        #                    data_type=DataType.sample)
        # _handler = BraggEdge(material=o_gui.get_element_selected())
        # _crystal_structure = _handler.metadata['crystal_structure'][o_gui.get_element_selected()]
        # _lattice = str(_handler.metadata['lattice'][o_gui.get_element_selected()])
        # self.parent.ui.lattice_parameter.setText(_lattice)
        # o_gui.set_crystal_structure(_crystal_structure)

        column_names = ["h", "k", "l", "\u03bb\u2090"]
        o_table = TableHandler(table_ui=self.parent.ui.pre_defined_tableWidget)
        o_table.set_column_names(column_names=column_names)
        o_table = TableHandler(table_ui=self.parent.ui.method1_tableWidget)
        o_table.set_column_names(column_names=column_names)

        self.parent.ui.method1_tableWidget.setEnabled(True)

        imgpix = QPixmap(error_icon_file)
        self.parent.ui.user_defined_name_error.setPixmap(imgpix)
        self.parent.ui.method1_lattice_error.setPixmap(imgpix)
        self.parent.ui.user_defined_method1_table_error.setPixmap(imgpix)
        self.parent.ui.user_defined_method2_table_error.setPixmap(imgpix)

    def labels(self):
        # micros
        self.parent.ui.micro_s.setText("\u00b5s")
        # distance source detector
        self.parent.ui.distance_source_detector_label.setText("d<sub> source-detector</sub>")
        # delta lambda
        self.parent.ui.delta_lambda_label.setText("\u0394\u03bb:")
        # Angstroms
        self.parent.ui.pre_defined_lattice_units.setText("\u212b")
        self.parent.ui.method1_lattice_units.setText("\u212b")

        # # tab 4
        # self.parent.ui.analysis_tab_arrow_label1.setStyleSheet(f"background-image: {right_blue_arrow}")
        # self.parent.ui.analysis_tab_arrow_label1.resize(200, 200)

    def general_init_pyqtgrpah(
        self,
        roi_function,
        base_widget,
        add_function,
        mean_function,
        file_index_function,
        tof_function,
        lambda_function,
        scroll_bar_function,
    ):
        area = DockArea()
        area.setVisible(False)
        d1 = Dock("Image Preview", size=(200, 200))
        d2 = Dock("Bragg Edge", size=(200, 200))

        area.addDock(d1, "top")
        area.addDock(d2, "bottom")

        preview_widget = pg.GraphicsLayoutWidget()
        pg.setConfigOptions(antialias=True)  # this improves the display

        vertical_layout = QVBoxLayout()
        preview_widget.setLayout(vertical_layout)

        # image view
        image_view = pg.ImageView(view=pg.PlotItem())
        image_view.ui.roiBtn.hide()
        image_view.ui.menuBtn.hide()
        roi = Roi.get_default_roi()
        # image_view.addItem(roi)
        roi.sigRegionChanged.connect(roi_function)

        roi_editor_button = QPushButton("ROI editor ...")
        roi_editor_button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        roi_editor_button.pressed.connect(self.parent.roi_editor_button_clicked)
        line_layout = QHBoxLayout()
        line_layout.addWidget(roi_editor_button)

        add_button = QRadioButton()
        add_button.setText("Add")
        add_button.setChecked(False)
        add_button.released.connect(add_function)
        line_layout.addWidget(add_button)

        mean_button = QRadioButton()
        mean_button.setText("Mean")
        mean_button.setChecked(True)
        mean_button.released.connect(mean_function)
        line_layout.addWidget(mean_button)

        top_widget = QWidget()
        top_widget.setLayout(line_layout)

        top_right_widget = QWidget()
        vertical = QVBoxLayout()
        vertical.addWidget(top_widget)

        vertical.addWidget(image_view)
        top_right_widget.setLayout(vertical)
        d1.addWidget(top_right_widget)

        # bragg edge plot
        bragg_edge_plot = pg.PlotWidget(title="")
        bragg_edge_plot.plot()

        # bragg_edge_plot.setLabel("top", "")
        # p1 = bragg_edge_plot.plotItem
        # p1.layout.removeItem(p1.getAxis('top'))
        # caxis = CustomAxis(orientation='top', parent=p1)
        # caxis.setLabel('')
        # caxis.linkToView(p1.vb)
        # p1.layout.addItem(caxis, 1, 1)
        caxis = None

        # add file_index, TOF, Lambda x-axis buttons
        hori_layout = QHBoxLayout()
        button_widgets = QWidget()
        button_widgets.setLayout(hori_layout)

        # file index
        file_index_button = QRadioButton()
        file_index_button.setText("File Index")
        file_index_button.setChecked(True)
        file_index_button.pressed.connect(file_index_function)

        # tof
        tof_button = QRadioButton()
        tof_button.setText("TOF")
        tof_button.pressed.connect(tof_function)

        # lambda
        lambda_button = QRadioButton()
        lambda_button.setText("\u03bb")
        lambda_button.pressed.connect(lambda_function)

        spacer1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        hori_layout.addItem(spacer1)
        hori_layout.addWidget(file_index_button)
        hori_layout.addWidget(tof_button)
        hori_layout.addWidget(lambda_button)
        spacer2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        hori_layout.addItem(spacer2)

        # hkl horizontal scroll widget
        scroll_hori_layout = QHBoxLayout()
        scroll_label = QLabel("Range of material Bragg peaks displayed:")
        scroll_hori_layout.addWidget(scroll_label)
        hori_scroll_widget = QScrollBar(QtCore.Qt.Horizontal)
        hori_scroll_widget.valueChanged.connect(scroll_bar_function)
        hori_scroll_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        scroll_hori_layout.addWidget(hori_scroll_widget)
        scroll_hori_widget = QWidget()
        scroll_hori_widget.setLayout(scroll_hori_layout)

        widget_vertical_layout = QVBoxLayout()
        widget_vertical_layout.addWidget(button_widgets)
        widget_vertical_layout.addWidget(scroll_hori_widget)

        bottom_widgets = QWidget()
        bottom_widgets.setLayout(widget_vertical_layout)

        d2.addWidget(bragg_edge_plot)
        d2.addWidget(bottom_widgets)

        vertical_layout.addWidget(area)
        base_widget.setLayout(vertical_layout)

        return [
            area,
            image_view,
            roi,
            bragg_edge_plot,
            caxis,
            roi_editor_button,
            add_button,
            mean_button,
            file_index_button,
            tof_button,
            lambda_button,
            scroll_label,
            hori_scroll_widget,
        ]

    def pyqtgraph(self):
        # sample
        [
            self.parent.ui.area,
            self.parent.ui.image_view,
            self.parent.ui.image_view_roi,
            self.parent.ui.bragg_edge_plot,
            self.parent.ui.caxis,
            self.parent.ui.roi_editor_button,
            self.parent.ui.roi_add_button,
            self.parent.ui.roi_mean_button,
            file_index_button,
            tof_button,
            lambda_button,
            self.parent.hkl_scrollbar_ui["label"][DataType.sample],
            self.parent.hkl_scrollbar_ui["widget"][DataType.sample],
        ] = self.general_init_pyqtgrpah(
            self.parent.roi_image_view_changed,
            self.parent.ui.preview_widget,
            self.parent.roi_algorithm_is_add_clicked,
            self.parent.roi_algorithm_is_mean_clicked,
            self.parent.file_index_xaxis_button_clicked,
            self.parent.tof_xaxis_button_clicked,
            self.parent.lambda_xaxis_button_clicked,
            self.parent.sample_hkl_scrollbar_changed,
        )

        self.parent.list_roi_id["sample"].append(self.parent.ui.image_view_roi)
        self.parent.xaxis_button_ui["sample"]["tof"] = tof_button
        self.parent.xaxis_button_ui["sample"]["file_index"] = file_index_button
        self.parent.xaxis_button_ui["sample"]["lambda"] = lambda_button

        # ob
        [
            self.parent.ui.ob_area,
            self.parent.ui.ob_image_view,
            self.parent.ui.ob_image_view_roi,
            self.parent.ui.ob_bragg_edge_plot,
            self.parent.ui.ob_caxis,
            self.parent.ui.ob_roi_editor_button,
            self.parent.ui.ob_roi_add_button,
            self.parent.ui.ob_roi_mean_button,
            file_index_button,
            tof_button,
            lambda_button,
            self.parent.hkl_scrollbar_ui["label"][DataType.ob],
            self.parent.hkl_scrollbar_ui["widget"][DataType.ob],
        ] = self.general_init_pyqtgrpah(
            self.parent.roi_ob_image_view_changed,
            self.parent.ui.ob_preview_widget,
            self.parent.ob_roi_algorithm_is_add_clicked,
            self.parent.ob_roi_algorithm_is_mean_clicked,
            self.parent.ob_file_index_xaxis_button_clicked,
            self.parent.ob_tof_xaxis_button_clicked,
            self.parent.ob_lambda_xaxis_button_clicked,
            self.parent.ob_hkl_scrollbar_changed,
        )

        self.parent.list_roi_id["ob"].append(self.parent.ui.ob_image_view_roi)
        self.parent.xaxis_button_ui["ob"]["tof"] = tof_button
        self.parent.xaxis_button_ui["ob"]["file_index"] = file_index_button
        self.parent.xaxis_button_ui["ob"]["lambda"] = lambda_button

        # normalized
        [
            self.parent.ui.normalized_area,
            self.parent.ui.normalized_image_view,
            self.parent.ui.normalized_image_view_roi,
            self.parent.ui.normalized_bragg_edge_plot,
            self.parent.ui.normalized_caxis,
            self.parent.ui.normalized_roi_editor_button,
            self.parent.ui.normalized_roi_add_button,
            self.parent.ui.normalized_roi_mean_button,
            file_index_button1,
            tof_button1,
            lambda_button1,
            self.parent.hkl_scrollbar_ui["label"][DataType.normalized],
            self.parent.hkl_scrollbar_ui["widget"][DataType.normalized],
        ] = self.general_init_pyqtgrpah(
            self.parent.roi_normalized_image_view_changed,
            self.parent.ui.normalized_preview_widget,
            self.parent.normalized_roi_algorithm_is_add_clicked,
            self.parent.normalized_roi_algorithm_is_mean_clicked,
            self.parent.normalized_file_index_xaxis_button_clicked,
            self.parent.normalized_tof_xaxis_button_clicked,
            self.parent.normalized_lambda_xaxis_button_clicked,
            self.parent.normalized_hkl_scrollbar_changed,
        )

        self.parent.list_roi_id["normalized"].append(self.parent.ui.normalized_image_view_roi)
        self.parent.xaxis_button_ui["normalized"]["tof"] = tof_button1
        self.parent.xaxis_button_ui["normalized"]["file_index"] = file_index_button1
        self.parent.xaxis_button_ui["normalized"]["lambda"] = lambda_button1

    def icons(self):
        # reset buttons
        preview_icon = QIcon(preview_file)
        self.parent.ui.preview_time_spectra_button.setIcon(preview_icon)
        self.parent.ui.preview_time_spectra_normalized_button.setIcon(preview_icon)

        # tab 6 - top buttons

        rotate_icon = QIcon(rotate_image)
        self.parent.ui.rotate_pushButton.setIcon(rotate_icon)
        self.parent.ui.rotate_pushButton.setIconSize(QSize(tab6_top_button_width, tab6_top_button_height))

        tof_combine_icon = QIcon(tof_combine_image)
        self.parent.ui.tof_combine_pushButton.setIcon(tof_combine_icon)
        self.parent.ui.tof_combine_pushButton.setIconSize(QSize(tab6_top_button_width, tab6_top_button_height))

        tof_binning_icon = QIcon(tof_binning_image)
        self.parent.ui.tof_binning_pushButton.setIcon(tof_binning_icon)
        self.parent.ui.tof_binning_pushButton.setIconSize(QSize(tab6_top_button_width, tab6_top_button_height))

        # tab6 - bottom buttons

        pixel_binning_icon = QIcon(pixel_binning_image)
        self.parent.ui.roi_pixel_binning_pushButton.setIcon(pixel_binning_icon)
        self.parent.ui.roi_pixel_binning_pushButton.setIconSize(
            QSize(tab6_bottom_buttom_width, tab6_bottom_button_height)
        )

        fitting_icon = QIcon(fitting_image)
        self.parent.ui.fitting_pushButton.setIcon(fitting_icon)
        self.parent.ui.fitting_pushButton.setIconSize(QSize(tab6_bottom_buttom_width, tab6_bottom_button_height))

        strain_mapping_icon = QIcon(strain_mapping_image)
        self.parent.ui.strain_mapping_pushButton.setIcon(strain_mapping_icon)
        self.parent.ui.strain_mapping_pushButton.setIconSize(QSize(tab6_bottom_buttom_width, tab6_bottom_button_height))



#!/usr/bin/env python
"""Presenter for the normalization settings view."""

import logging
from typing import Any, Dict

from ibeatles.app.models.normalization_settings_model import NormalizationSettingsModel
from ibeatles.app.ui.normalization_settings_view import NormalizationSettingsView
from ibeatles.core.config import NormalizationConfig


class NormalizationSettingsPresenter:
    """
    Presenter for the normalization settings.

    This class acts as an intermediary between the NormalizationSettingsModel
    and NormalizationSettingsView, handling user interactions and updating
    the model and view accordingly.

    Parameters
    ----------
    parent : QWidget, optional
        The parent widget for the view.

    Attributes
    ----------
    parent : QWidget
        The parent widget for the view.
    model : NormalizationSettingsModel
        The model containing the normalization settings data.
    view : NormalizationSettingsView
        The view displaying the normalization settings UI.
    """

    def __init__(self, parent=None):
        self.parent = parent
        self.model = NormalizationSettingsModel()
        self.view = NormalizationSettingsView(parent)
        self.connect_signals()

    def connect_signals(self):
        """Connect view signals to their respective slots."""
        self.view.settings_changed.connect(self.update_model_from_view)

    def load_settings(self, config: NormalizationConfig = None, old_config: Dict[str, Any] = None):
        """
        Load settings into the model and update the view.

        Parameters
        ----------
        config : NormalizationConfig, optional
            The new configuration to load.
        old_config : Dict[str, Any], optional
            The old configuration format to load.

        Raises
        ------
        ValueError
            If neither config nor old_config is provided.
        """
        if config:
            self.model.update_from_config(config)
        elif old_config:
            self.model.update_from_old_config(old_config)
        else:
            raise ValueError("Either config or old_config must be provided")

        self.update_view_from_model()

    def update_view_from_model(self):
        """Update the view with the current model state."""
        config = self.model.get_config()
        self.view.set_settings(config)

    def update_model_from_view(self):
        """Update the model with the current view state."""
        view_config = self.view.get_settings()
        self.model.update_from_config(view_config)

    def get_config(self) -> NormalizationConfig:
        """
        Get the current configuration from the model.

        Returns
        -------
        NormalizationConfig
            The current normalization configuration.
        """
        self.update_model_from_view()
        return self.model.get_config()

    def get_old_config(self) -> Dict[str, Any]:
        """
        Get the current configuration in the old format.

        Returns
        -------
        Dict[str, Any]
            The current configuration in the old dictionary format.
        """
        self.update_model_from_view()
        return self.model.get_old_config()

    def show_view(self):
        """Display the normalization settings view."""
        if self.view.exec_() == self.view.Accepted:
            self.update_model_from_view()
            if self.parent and hasattr(self.parent, "session_dict"):
                self.parent.session_dict["reduction"] = self.model.get_old_config()
            # logging
            logging.debug("Settings accepted. Current NormalizationConfig:")
            logging.debug(self.model.config.model_dump_json(indent=2))
